@@ -6,59 +6,19 @@ import random
 import re
 import string
 import sys
-import textwrap
 from typing import Dict, List, Optional, Tuple, Union  # noqa: F401
 
-import pycloudlib  # type: ignore
+import pycloudlib  # type: ignore  # noqa: F401
 from behave.model import Feature, Scenario
 from behave.runner import Context
 
 import features.cloud as cloud
-from features.util import InstallationSource, build_debs, lxc_get_property
-
-ALL_SUPPORTED_SERIES = ["bionic", "focal", "xenial"]
-
-UA_PPA_TEMPLATE = "http://ppa.launchpad.net/ua-client/{}/ubuntu"
-DEFAULT_UA_PPA_KEYID = "6E34E7116C0BC933"
-
-USERDATA_BLOCK_AUTO_ATTACH_IMG = """\
-#cloud-config
-[bootcmd, once]:
- - cp /usr/bin/ua /usr/bin/ua.orig
- - 'echo "#!/bin/sh\ndate >> /root/ua-calls\n" > /usr/bin/ua'
- - chmod 755 /usr/bin/ua
-"""
-
-# we can't use write_files because it will clash with the
-# lxd vendor-data write_files that is necessary to set up
-# the lxd_agent on some releases
-USERDATA_RUNCMD_PIN_PPA = """\
-runcmd:
-  - "printf \\"Package: *\\nPin: release o=LP-PPA-ua-client-{}\\nPin-Priority: 1001\\n\\" > /etc/apt/preferences.d/uaclient"
-"""  # noqa: E501
-
-USERDATA_RUNCMD_ENABLE_PROPOSED = """
-runcmd:
-  - printf \"deb http://archive.ubuntu.com/ubuntu/ {series}-proposed main\" > /etc/apt/sources.list.d/uaclient-proposed.list
-  - "printf \\"Package: *\\nPin: release a={series}-proposed\\nPin-Priority: 400\\n\\" > /etc/apt/preferences.d/lower-proposed"
-  - "printf \\"Package: ubuntu-advantage-tools\\nPin: release a={series}-proposed\\nPin-Priority: 1001\\n\\" > /etc/apt/preferences.d/uaclient-proposed"
-"""  # noqa: E501
-
-USERDATA_APT_SOURCE_PPA = """\
-apt:
-  sources:
-    ua-tools-ppa:
-        source: deb {ppa_url} $RELEASE main
-        keyid: {ppa_keyid}
-"""
-
-PROCESS_LOG_TMPL = """\
-returncode: {}
-stdout:
-{stdout}
-stderr:
-{stderr}
-"""
+from features.util import (
+    SUT,
+    InstallationSource,
+    lxc_get_property,
+    process_template_vars,
+)
 
 PROCESS_LOG_TMPL = """\
 returncode: {returncode}
@@ -128,11 +88,10 @@ class UAClientBehaveConfig:
         "private_key_file",
         "private_key_name",
         "reuse_image",
-        "debs_path",
         "artifact_dir",
         "install_from",
         "custom_ppa",
-        "custom_ppa_keyid",
+        "debs_path",
         "userdata_file",
         "check_version",
         "sbuild_chroot",
@@ -152,26 +111,25 @@ class UAClientBehaveConfig:
     def __init__(
         self,
         *,
-        cloud_credentials_path: str = None,
+        cloud_credentials_path: Optional[str] = None,
         image_clean: bool = True,
         destroy_instances: bool = True,
         ephemeral_instance: bool = False,
         snapshot_strategy: bool = False,
         machine_type: str = "lxd.container",
-        private_key_file: str = None,
+        private_key_file: Optional[str] = None,
         private_key_name: str = "uaclient-integration",
-        reuse_image: str = None,
-        contract_token: str = None,
-        contract_token_staging: str = None,
-        contract_token_staging_expired: str = None,
-        debs_path: str = None,
-        artifact_dir: str = None,
+        reuse_image: Optional[str] = None,
+        contract_token: Optional[str] = None,
+        contract_token_staging: Optional[str] = None,
+        contract_token_staging_expired: Optional[str] = None,
+        artifact_dir: Optional[str] = None,
         install_from: InstallationSource = InstallationSource.DAILY,
-        custom_ppa: str = None,
-        custom_ppa_keyid: str = None,
-        userdata_file: str = None,
-        check_version: str = None,
-        sbuild_chroot: str = None,
+        custom_ppa: Optional[str] = None,
+        debs_path: Optional[str] = None,
+        userdata_file: Optional[str] = None,
+        check_version: Optional[str] = None,
+        sbuild_chroot: Optional[str] = None,
         cmdline_tags: List = []
     ) -> None:
         # First, store the values we've detected
@@ -188,11 +146,10 @@ class UAClientBehaveConfig:
         self.private_key_name = private_key_name
         self.reuse_image = reuse_image
         self.cmdline_tags = cmdline_tags
-        self.debs_path = debs_path
         self.artifact_dir = artifact_dir
         self.install_from = install_from
         self.custom_ppa = custom_ppa
-        self.custom_ppa_keyid = custom_ppa_keyid
+        self.debs_path = debs_path
         self.userdata_file = userdata_file
         self.check_version = check_version
         self.sbuild_chroot = sbuild_chroot
@@ -204,6 +161,19 @@ class UAClientBehaveConfig:
             ]
         )
         # Next, perform any required validation
+        if install_from == InstallationSource.CUSTOM and not custom_ppa:
+            logging.error(
+                "UACLIENT_BEHAVE_INSTALL_FROM is set to 'custom', "
+                "but missing UACLIENT_BEHAVE_CUSTOM_PPA"
+            )
+            sys.exit(1)
+        if install_from == InstallationSource.PREBUILT and not debs_path:
+            logging.error(
+                "UACLIENT_BEHAVE_INSTALL_FROM is set to 'prebuilt', "
+                "but missing UACLIENT_BEHAVE_DEBS_PATH"
+            )
+            sys.exit(1)
+
         if self.reuse_image is not None:
             if self.image_clean:
                 print(" Reuse_image specified, it will not be deleted.")
@@ -254,6 +224,7 @@ class UAClientBehaveConfig:
                 tag=timed_job_tag,
                 timestamp_suffix=False,
             )
+            self.cloud = "aws"
         elif "azure" in self.machine_type:
             self.cloud_manager = cloud.Azure(
                 machine_type=self.machine_type,
@@ -261,6 +232,7 @@ class UAClientBehaveConfig:
                 tag=timed_job_tag,
                 timestamp_suffix=False,
             )
+            self.cloud = "azure"
         elif "gcp" in self.machine_type:
             self.cloud_manager = cloud.GCP(
                 machine_type=self.machine_type,
@@ -268,16 +240,19 @@ class UAClientBehaveConfig:
                 tag=timed_job_tag,
                 timestamp_suffix=False,
             )
+            self.cloud = "gcp"
         elif "lxd.vm" in self.machine_type:
             self.cloud_manager = cloud.LXDVirtualMachine(
                 machine_type=self.machine_type,
                 cloud_credentials_path=self.cloud_credentials_path,
             )
+            self.cloud = "lxd.vm"
         else:
             self.cloud_manager = cloud.LXDContainer(
                 machine_type=self.machine_type,
                 cloud_credentials_path=self.cloud_credentials_path,
             )
+            self.cloud = "lxd"
 
         self.cloud_api = self.cloud_manager.api
 
@@ -333,6 +308,18 @@ class UAClientBehaveConfig:
 def before_all(context: Context) -> None:
     """behave will invoke this before anything else happens."""
     context.config.setup_logging()
+    if logging.getLogger().level == logging.DEBUG:
+        # The AWS boto libraries are very very very verbose
+        # We pretty much never want their debug level messages,
+        # but we do want to be able to use the debug log level
+        # in our code. So we bump their loggers up to info
+        # when we set debug at the cli with --logging-level=debug
+        logging.warn(
+            "Setting AWS botocore and boto3 loggers to INFO to avoid"
+            " extra verbose logs"
+        )
+        logging.getLogger("botocore").setLevel(logging.INFO)
+        logging.getLogger("boto3").setLevel(logging.INFO)
     userdata = context.config.userdata
     if userdata:
         logging.debug("Userdata key / value pairs:")
@@ -342,9 +329,10 @@ def before_all(context: Context) -> None:
             print("   - {} = {}".format(key, value))
     context.series_image_name = {}
     context.series_reuse_image = ""
-    context.reuse_container = {}
     context.config = UAClientBehaveConfig.from_environ(context.config)
     context.config.cloud_manager.manage_ssh_key()
+    context.snapshots = {}
+    context.machines = {}
 
     if context.config.reuse_image:
         series = lxc_get_property(
@@ -364,26 +352,6 @@ def before_all(context: Context) -> None:
             print(" Could not check image series. It will not be used. ")
             context.config.reuse_image = None
 
-    if userdata.get("reuse_container"):
-        inst = context.config.cloud_api.get_instance(
-            userdata.get("reuse_container")
-        )
-        codename = inst.execute(
-            ["grep", "UBUNTU_CODENAME", "/etc/os-release"]
-        ).strip()
-        [_, series] = codename.split("=")
-
-        context.reuse_container = {series: userdata.get("reuse_container")}
-        print(
-            textwrap.dedent(
-                """
-            You are providing a {series} container. Make sure you are running
-            this series tests. For instance: --tags=series.{series}""".format(
-                    series=series
-                )
-            )
-        )
-
 
 def _should_skip_tags(context: Context, tags: List) -> str:
     """Return a reason if a feature or scenario should be skipped"""
@@ -422,11 +390,8 @@ def before_feature(context: Context, feature: Feature):
 
 
 def before_scenario(context: Context, scenario: Scenario):
-    """
-    In this function, we launch a container, install ubuntu-advantage-tools and
-    then capture an image. This image is then reused by each scenario, reducing
-    test execution time.
-    """
+    context.stored_vars = {}
+
     reason = _should_skip_tags(context, scenario.effective_tags)
     if reason:
         scenario.skip(reason=reason)
@@ -450,21 +415,28 @@ def before_scenario(context: Context, scenario: Scenario):
             )
             return
 
-    if "uses.config.check_version" in scenario.effective_tags:
-        # before_step doesn't execute early enough to modify the step
-        # so we perform the version step surgery here
-        for step in scenario.steps:
-            if step.text:
-                step.text = step.text.replace(
-                    "{UACLIENT_BEHAVE_CHECK_VERSION}",
-                    context.config.check_version,
-                )
+    # before_step doesn't execute early enough to modify the step
+    # so we perform step text surgery here
+    # Also, logging capture is not set up when before_scenario is called,
+    # so if you call logging.info here, then the root logger gets configured
+    # and it messes up all the future behave logging capture machinery.
+    # See https://github.com/behave/behave/blob/v1.2.6/behave/model.py#L700
+    # But we want to log the replacement we're making, so we use the behave
+    # logger and warning log_level to make sure it gets included.
+    logger = logging.getLogger("behave.before_scenario.process_template_vars")
+    for step in scenario.steps:
+        if step.text:
+            step.text = process_template_vars(
+                context, step.text, logger_fn=logger.warn
+            )
 
 
 FAILURE_FILES = (
     "/etc/ubuntu-advantage/uaclient.log",
     "/var/log/cloud-init.log",
     "/var/log/ubuntu-advantage.log",
+    "/var/log/ubuntu-advantage-daemon.log",
+    "/var/log/ubuntu-advantage-timer.log",
     "/var/lib/cloud/instance/user-data.txt",
     "/var/lib/cloud/instance/vendor-data.txt",
 )
@@ -472,7 +444,7 @@ FAILURE_CMDS = {
     "ua-version": ["pro", "version"],
     "cloud-init-analyze": ["cloud-init", "analyze", "show"],
     "cloud-init.status": ["cloud-init", "status", "--long"],
-    "status.json": ["pro", "status", "--all", "--format=json"],
+    "status.yaml": ["pro", "status", "--all", "--format=yaml"],
     "journal.log": ["journalctl", "-b", "0"],
     "systemd-analyze-blame": ["systemd-analyze", "blame"],
     "systemctl-status": ["systemctl", "status"],
@@ -486,6 +458,16 @@ FAILURE_CMDS = {
         "status",
         "ua-reboot-cmds.service",
     ],
+    "systemctl-status-ubuntu-advantage": [
+        "systemctl",
+        "status",
+        "ubuntu-advantage.service",
+    ],
+    "systemctl-status-apt-news": [
+        "systemctl",
+        "status",
+        "apt-news.service",
+    ],
 }
 
 
@@ -514,7 +496,7 @@ def after_step(context, step):
                     )
                 )
 
-        if hasattr(context, "instance"):
+        if hasattr(context, "machines") and SUT in context.machines:
             if not os.path.exists(artifacts_dir):
                 os.makedirs(artifacts_dir)
             for log_file in FAILURE_FILES:
@@ -525,7 +507,7 @@ def after_step(context, step):
                     "-- pull instance:{} {}".format(log_file, artifact_file)
                 )
                 try:
-                    result = context.instances["uaclient"].execute(
+                    result = context.machines[SUT].instance.execute(
                         ["cat", log_file], use_sudo=True
                     )
                     content = result.stdout if result.ok else ""
@@ -534,7 +516,7 @@ def after_step(context, step):
                 with open(artifact_file, "w") as stream:
                     stream.write(content)
             for artifact_file, cmd in FAILURE_CMDS.items():
-                result = context.instances["uaclient"].execute(
+                result = context.machines[SUT].instance.execute(
                     cmd, use_sudo=True
                 )
                 artifact_file = os.path.join(artifacts_dir, artifact_file)
@@ -553,278 +535,24 @@ def after_all(context):
             else:
                 context.config.cloud_api.delete_image(image)
 
-
-def capture_container_as_image(
-    container_id: str, image_name: str, cloud_api: pycloudlib.cloud.BaseCloud
-) -> str:
-    """Capture a container as an image.
-
-    :param container_id:
-        The id of the container to be captured.  Note that this container
-        will be stopped.
-    :param image_name:
-        The name under which the image should be published.
-    :param cloud_api: Optional pycloud BaseCloud api for applicable
-        machine_types.
-    """
-    logging.info(
-        "--- Creating  base image snapshot from vm {}".format(container_id)
-    )
-    inst = cloud_api.get_instance(container_id)
-    return cloud_api.snapshot(instance=inst)
-
-
-def build_debs_from_sbuild(context: Context, series: str) -> List[str]:
-    """Create a chroot and build the package using sbuild
-
-
-    Will stop the development instance after deb build succeeds.
-
-    :return: A list of paths to applicable deb files published.
-    """
-    deb_paths = []
-
-    if context.config.debs_path:
-        logging.info(
-            "--- Checking if debs can be reused in {}".format(
-                context.config.debs_path
-            )
-        )
-        debs_path = context.config.debs_path
-        if os.path.isdir(debs_path):
-            deb_paths = [
-                os.path.join(debs_path, deb_file)
-                for deb_file in os.listdir(debs_path)
-                if series in deb_file
-            ]
-
-    if len(deb_paths):
-        logging.info("--- Reusing debs: {}".format(", ".join(deb_paths)))
-    else:
-        logging.info(
-            "--- Could not find any debs to reuse. Building it locally"
-        )
-        deb_paths = build_debs(
-            series=series,
-            chroot=context.config.sbuild_chroot,
-        )
-
-    if "pro" in context.config.machine_type:
-        return deb_paths
-    # Redact ubuntu-advantage-pro deb as inapplicable
-    return [deb_path for deb_path in deb_paths if "pro" not in deb_path]
-
-
-def create_instance_with_uat_installed(
-    context: Context,
-    series: str,
-    name: str,
-    custom_user_data: Optional[str] = None,
-) -> pycloudlib.instance.BaseInstance:
-    """Create a given series lxd image with ubuntu-advantage-tools installed
-
-    This will launch a container, install ubuntu-advantage-tools, and publish
-    the image. The image's name is stored in context.series_image_name for
-    use within step code.
-
-    :param context:
-        A `behave.runner.Context`;  this will have `series.image_name` set on
-        it.
-    :param series:
-       A string representing the series name to create
-    :param name:
-       A string representing the instance name
-    :param custom_user_data:
-       A string representing custom userdata to be added to the instance
-
-    :return: A pycloudlib Instance
-    """
-
-    if series in context.reuse_container:
-        logging.info(
-            "\n Reusing the existing container: ",
-            context.reuse_container[series],
-        )
-        return
-
-    user_data = _get_user_data_for_instance(context.config, series)
-
-    if custom_user_data:
-        prefix = "" if user_data else "#cloud-config"
-        user_data += "{}\n{}".format(prefix, custom_user_data)
-
-    logging.info(
-        "--- Launching VM to create a base image with ubuntu-advantage"
-    )
-    inst = context.config.cloud_manager.launch(
-        instance_name=name, series=series, user_data=user_data
-    )
-    instance_id = context.config.cloud_manager.get_instance_id(inst)
-
-    deb_paths = None
-    if context.config.install_from is InstallationSource.LOCAL:
-        deb_paths = build_debs_from_sbuild(context, series)
-
-    _install_uat_in_container(
-        instance_id,
-        series=series,
-        config=context.config,
-        machine_type=context.config.machine_type,
-        deb_paths=deb_paths,
-    )
-
-    return inst
-
-
-def _get_user_data_for_instance(
-    config: UAClientBehaveConfig, series: str
-) -> str:
-    if config.install_from in (
-        InstallationSource.ARCHIVE,
-        InstallationSource.LOCAL,
-    ):
-        return ""
-
-    user_data = "#cloud-config\n"
-
-    if "pro" in config.machine_type:
-        user_data = USERDATA_BLOCK_AUTO_ATTACH_IMG
-
-    if config.userdata_file and os.path.exists(config.userdata_file):
-        with open(config.userdata_file, "r") as stream:
-            user_data = stream.read()
-
-    if config.install_from is InstallationSource.PROPOSED:
-        user_data += USERDATA_RUNCMD_ENABLE_PROPOSED.format(series=series)
-        return user_data
-
-    if config.install_from is InstallationSource.DAILY:
-        user_data += USERDATA_RUNCMD_PIN_PPA.format("daily")
-        ppa = UA_PPA_TEMPLATE.format("daily")
-        ppa_keyid = DEFAULT_UA_PPA_KEYID
-    elif config.install_from is InstallationSource.STAGING:
-        user_data += USERDATA_RUNCMD_PIN_PPA.format("staging")
-        ppa = UA_PPA_TEMPLATE.format("staging")
-        ppa_keyid = DEFAULT_UA_PPA_KEYID
-    elif config.install_from is InstallationSource.STABLE:
-        user_data += USERDATA_RUNCMD_PIN_PPA.format("stable")
-        ppa = UA_PPA_TEMPLATE.format("stable")
-        ppa_keyid = DEFAULT_UA_PPA_KEYID
-    elif config.install_from is InstallationSource.CUSTOM:
-        if not config.custom_ppa or not config.custom_ppa_keyid:
+    if context.config.destroy_instances:
+        try:
+            key_pair = context.config.cloud_manager.api.key_pair
+            os.remove(key_pair.private_key_path)
+            os.remove(key_pair.public_key_path)
+        except Exception as e:
             logging.error(
-                "UACLIENT_BEHAVE_INSTALL_FROM is set to 'custom', "
-                + "but missing UACLIENT_BEHAVE_CUSTOM_PPA or "
-                + "UACLIENT_BEHAVE_CUSTOM_PPA_KEYID"
+                "Failed to delete instance ssh keys:\n{}".format(str(e))
             )
-            sys.exit(1)
-        ppa = config.custom_ppa
-        if ppa.startswith("ppa:"):
-            ppa = ppa.replace("ppa:", "http://ppa.launchpad.net/") + "/ubuntu"
-        ppa_keyid = config.custom_ppa_keyid or ""
 
-    user_data += USERDATA_APT_SOURCE_PPA.format(
-        ppa_url=ppa, ppa_keyid=ppa_keyid
-    )
-
-    logging.info("-- user data:\n" + user_data)
-    return user_data
-
-
-def _install_uat_in_container(
-    container_id: str,
-    series: str,
-    config: UAClientBehaveConfig,
-    machine_type: str,
-    deb_paths: Optional[List[str]] = None,
-) -> None:
-    """Install ubuntu-advantage-tools into the specified container
-
-    :param container_id:
-        The id of the container into which ubuntu-advantage-tools should be
-        installed.
-    :param series: The name of the series that is being used
-    :param config: UAClientBehaveConfig
-    :param deb_paths: Optional paths to local deb files we need to install
-    """
-    cmds = ["systemctl is-system-running --wait"]  # type:  List[str]
-
-    if deb_paths is None:
-        deb_paths = []
-
-    if deb_paths:
-        cmds.append("sudo apt-get update -qqy")
-
-        deb_files = []
-        inst = config.cloud_api.get_instance(container_id)
-
-        for deb_file in deb_paths:
-            if "pro" in deb_file and "pro" not in config.machine_type:
-                continue
-
-            deb_name = os.path.basename(deb_file)
-            deb_files.append("/tmp/" + deb_name)
-            inst.push_file(deb_file, "/tmp/" + deb_name)
-
-        cmds.append(
-            " ".join(
-                [
-                    "sudo",
-                    "DEBIAN_FRONTEND=noninteractive",
-                    "apt-get",
-                    "install",
-                    "-y",
-                    "--allow-downgrades",
-                    '-o Dpkg::Options::="--force-confdef"',
-                    '-o Dpkg::Options::="--force-confold"',
-                ]
-                + deb_files
+    if "builder" in context.snapshots:
+        try:
+            context.config.cloud_manager.api.delete_image(
+                context.snapshots["builder"]
             )
-        )
-    else:
-        ua_pkg = ["ubuntu-advantage-tools"]
-        if "pro" in machine_type:
-            ua_pkg.append("ubuntu-advantage-pro")
-
-        cmds.append("sudo apt-get update")
-        cmds.append(
-            " ".join(
-                [
-                    "sudo",
-                    "DEBIAN_FRONTEND=noninteractive",
-                    "apt-get",
-                    "install",
-                    "-y",
-                    "--allow-downgrades",
-                    '-o Dpkg::Options::="--force-confdef"',
-                    '-o Dpkg::Options::="--force-confold"',
-                ]
-                + ua_pkg
-            )
-        )
-
-    if "pro" in config.machine_type:
-        features = "features:\n  disable_auto_attach: true\n"
-        conf_path = "/etc/ubuntu-advantage/uaclient.conf"
-        cmd = "printf '{}' > /var/tmp/uaclient.conf".format(features)
-        cmds.append('sh -c "{}"'.format(cmd))
-        cmds.append(
-            'sudo -- sh -c "cat /var/tmp/uaclient.conf >> {}"'.format(
-                conf_path
-            )
-        )
-        cmds.append("sudo pro status --wait")
-        cmds.append("sudo pro detach --assume-yes")
-
-    cmds.append("pro version")
-    instance = config.cloud_api.get_instance(container_id)
-    for cmd in cmds:  # type: ignore
-        result = instance.execute(cmd)
-        if result.failed:
-            logging.info(
-                "--- Failed {}: out {} err {}".format(
-                    cmd, result.stdout, result.stderr
+        except RuntimeError as e:
+            logging.error(
+                "Failed to delete image: {}\n{}".format(
+                    context.snapshots["builder"], str(e)
                 )
             )
-        elif "version" in cmd:
-            logging.info("--- " + result)
