@@ -1,15 +1,17 @@
+import datetime
 import hashlib
 import logging
 import os
+import re
 import shutil
 import subprocess
 import tempfile
-import textwrap
 from enum import Enum
-from typing import Iterable, List, Optional
+from typing import Callable, Iterable, List, Optional
 
 import yaml
 
+SUT = "system-under-test"
 LXC_PROPERTY_MAP = {
     "image": {"series": "properties.release", "machine_type": "Type"},
     "container": {"series": "image.release", "machine_type": "image.type"},
@@ -19,29 +21,11 @@ SOURCE_PR_TGZ = os.path.join(UA_TMP_DIR, "pr_source.tar.gz")
 SOURCE_PR_UNTAR_DIR = os.path.join(UA_TMP_DIR, "behave-ua-src")
 SBUILD_DIR = os.path.join(UA_TMP_DIR, "sbuild")
 UA_DEB_BUILD_CACHE = os.path.join(UA_TMP_DIR, "deb-cache")
-UA_DEBS = frozenset({"ubuntu-advantage-tools.deb", "ubuntu-advantage-pro.deb"})
-
-
-BUILD_FROM_TGZ = textwrap.dedent(
-    """\
-   #!/bin/bash
-   set -o xtrace
-   apt-get update
-   apt-get install make
-   cd /tmp
-   tar -zxf *gz
-   cd *ubuntu-advantage-client*
-   make deps
-   dpkg-buildpackage -us -uc
-   # Copy and rename versioned debs to /tmp/ubuntu-advantage-(tools|pro).deb
-   cp /tmp/ubuntu-advantage-tools*.deb /tmp/ubuntu-advantage-tools.deb
-   cp /tmp/ubuntu-advantage-pro*.deb /tmp/ubuntu-advantage-pro.deb
-   """
-)
 
 
 class InstallationSource(Enum):
     ARCHIVE = "archive"
+    PREBUILT = "prebuilt"
     LOCAL = "local"
     DAILY = "daily"
     STAGING = "staging"
@@ -155,6 +139,8 @@ def repo_state_hash(
                 break
         if exclude:
             continue
+        if os.path.isdir(fname):
+            continue
         with open(fname) as f:
             new_file_content += f.read()
 
@@ -290,22 +276,71 @@ class SafeLoaderWithoutDatetime(yaml.SafeLoader):
     }
 
 
-def cleanup_instance(context, instance_name):
-    def _cleanup_instance():
-        if not context.config.destroy_instances:
-            logging.info(
-                "--- Leaving instance running: {}".format(
-                    context.instances[instance_name].name
-                )
-            )
-            return
-        try:
-            context.instances[instance_name].delete(wait=False)
-        except RuntimeError as e:
-            logging.error(
-                "Failed to delete instance: {}\n{}".format(
-                    context.instances[instance_name].name, str(e)
-                )
-            )
+def _replace_and_log(s, old, new, logger_fn):
+    logger_fn('replacing "{}" with "{}"'.format(old, new))
+    return s.replace(old, new)
 
-    return _cleanup_instance
+
+def process_template_vars(
+    context, template: str, logger_fn: Optional[Callable] = None
+) -> str:
+    if logger_fn is None:
+        logger_fn = logging.info
+
+    processed_template = template
+
+    for match in re.finditer(r"\$behave_var{([\w\s\-\+]*)}", template):
+        args = match.group(1).split()
+        function_name = args[0]
+        if function_name == "version":
+            if context.config.check_version:
+                processed_template = _replace_and_log(
+                    processed_template,
+                    match.group(0),
+                    context.config.check_version,
+                    logger_fn,
+                )
+        elif function_name == "machine-ip":
+            if args[1] in context.machines:
+                processed_template = _replace_and_log(
+                    processed_template,
+                    match.group(0),
+                    context.machines[args[1]].instance.ip,
+                    logger_fn,
+                )
+        elif function_name == "cloud":
+            processed_template = _replace_and_log(
+                processed_template,
+                match.group(0),
+                context.config.cloud,
+                logger_fn,
+            )
+        elif function_name == "today":
+            dt = datetime.datetime.utcnow()
+            if len(args) == 2:
+                offset = int(args[1])
+                dt = dt + datetime.timedelta(days=offset)
+            dt_str = dt.strftime("%Y-%m-%dT00:00:00Z")
+            processed_template = _replace_and_log(
+                processed_template,
+                match.group(0),
+                dt_str,
+                logger_fn,
+            )
+        elif function_name == "contract_token_staging":
+            processed_template = _replace_and_log(
+                processed_template,
+                match.group(0),
+                context.config.contract_token_staging,
+                logger_fn,
+            )
+        elif function_name == "stored_var":
+            if context.stored_vars.get(args[1]):
+                processed_template = _replace_and_log(
+                    processed_template,
+                    match.group(0),
+                    context.stored_vars.get(args[1]),
+                    logger_fn,
+                )
+
+    return processed_template
