@@ -1,17 +1,29 @@
+import datetime
 import io
 import logging
 import sys
+from enum import Enum
 from typing import Any, Dict
 
 import mock
 import pytest
 
-from uaclient import event_logger
-from uaclient.config import UAConfig
-
 # We are doing this because we are sure that python3-apt comes with the distro,
 # but it cannot be installed in a virtual environment to be properly tested.
+# Those need to be mocked here, before importing our modules, so the pytest
+# virtualenv doesn't cry because it can't find the modules
+m_apt_pkg = mock.MagicMock()
 sys.modules["apt"] = mock.MagicMock()
+sys.modules["apt_pkg"] = m_apt_pkg
+
+# Useless try/except to make flake8 happy \_("/)_/
+try:
+    from uaclient import event_logger
+    from uaclient.config import UAConfig
+    from uaclient.files.notices import NoticeFileDetails
+    from uaclient.files.state_files import UserConfigData
+except ImportError:
+    raise
 
 
 @pytest.yield_fixture(scope="session", autouse=True)
@@ -44,6 +56,19 @@ def _warn_about_new_version():
         yield original
 
 
+@pytest.yield_fixture(scope="session", autouse=True)
+def util_we_are_currently_root():
+    """
+    A fixture that mocks util.we_are_currently_root for all tests.
+    Default to true as most tests need it to be true.
+    """
+    from uaclient.util import we_are_currently_root
+
+    original = we_are_currently_root
+    with mock.patch("uaclient.util.we_are_currently_root", return_value=True):
+        yield original
+
+
 @pytest.fixture
 def caplog_text(request):
     """
@@ -116,11 +141,13 @@ def FakeConfig(tmpdir):
             self,
             cfg_overrides={},
             features_override=None,
-            root_mode: bool = True,
         ) -> None:
             if not cfg_overrides.get("data_dir"):
                 cfg_overrides.update({"data_dir": tmpdir.strpath})
-            super().__init__(cfg_overrides, root_mode=root_mode)
+            super().__init__(
+                cfg_overrides,
+                user_config=UserConfigData(),
+            )
 
         @classmethod
         def for_attached_machine(
@@ -128,7 +155,7 @@ def FakeConfig(tmpdir):
             account_name: str = "test_account",
             machine_token: Dict[str, Any] = None,
             status_cache: Dict[str, Any] = None,
-            root_mode: bool = True,
+            effective_to: datetime.datetime = None,
         ):
             if not machine_token:
                 machine_token = {
@@ -139,7 +166,15 @@ def FakeConfig(tmpdir):
                         "accountInfo": {
                             "id": "acct-1",
                             "name": account_name,
-                            "createdAt": "2019-06-14T06:45:50Z",
+                            "createdAt": datetime.datetime(
+                                2019,
+                                6,
+                                14,
+                                6,
+                                45,
+                                50,
+                                tzinfo=datetime.timezone.utc,
+                            ),
                             "externalAccountIDs": [
                                 {"IDs": ["id1"], "origin": "AWS"}
                             ],
@@ -147,20 +182,49 @@ def FakeConfig(tmpdir):
                         "contractInfo": {
                             "id": "cid",
                             "name": "test_contract",
-                            "createdAt": "2020-05-08T19:02:26Z",
-                            "effectiveFrom": "2000-05-08T19:02:26Z",
-                            "effectiveTo": "2040-05-08T19:02:26Z",
+                            "createdAt": datetime.datetime(
+                                2020,
+                                5,
+                                8,
+                                19,
+                                2,
+                                26,
+                                tzinfo=datetime.timezone.utc,
+                            ),
+                            "effectiveFrom": datetime.datetime(
+                                2000,
+                                5,
+                                8,
+                                19,
+                                2,
+                                26,
+                                tzinfo=datetime.timezone.utc,
+                            ),
+                            "effectiveTo": datetime.datetime(
+                                2040,
+                                5,
+                                8,
+                                19,
+                                2,
+                                26,
+                                tzinfo=datetime.timezone.utc,
+                            ),
                             "resourceEntitlements": [],
                             "products": ["free"],
                         },
                     },
                 }
 
+            if effective_to:
+                machine_token["machineTokenInfo"]["contractInfo"][
+                    "effectiveTo"
+                ] = effective_to
+
             if not status_cache:
                 status_cache = {"attached": True}
 
-            config = cls(root_mode=root_mode)
-            config.machine_token_file.write(machine_token)
+            config = cls()
+            config.machine_token_file._machine_token = machine_token
             config.write_cache("status-cache", status_cache)
             return config
 
@@ -177,3 +241,29 @@ def event():
     event.reset()
 
     return event
+
+
+class FakeNotice(NoticeFileDetails, Enum):
+    a = NoticeFileDetails("01", "a", True, "notice_a")
+    a2 = NoticeFileDetails("03", "a2", True, "notice_a2")
+    b = NoticeFileDetails("02", "b2", False, "notice_b")
+
+
+@pytest.yield_fixture(autouse=True)
+def mock_notices_dir(tmpdir_factory):
+    perm_dir = tmpdir_factory.mktemp("notices")
+    temp_dir = tmpdir_factory.mktemp("temp_notices")
+    with mock.patch(
+        "uaclient.defaults.NOTICES_PERMANENT_DIRECTORY",
+        perm_dir.strpath,
+    ):
+        with mock.patch(
+            "uaclient.defaults.NOTICES_TEMPORARY_DIRECTORY",
+            temp_dir.strpath,
+        ):
+            yield
+
+
+@pytest.fixture
+def apt_pkg():
+    return m_apt_pkg
