@@ -50,7 +50,7 @@ class TestCommonCriteriaEntitlementUserFacingStatus:
                 "xenial",
                 "16.04 LTS (Xenial Xerus)",
                 "CC EAL2 is not available for platform arm64.\n"
-                "Supported platforms are: x86_64, ppc64le, s390x.",
+                "Supported platforms are: amd64, ppc64el, s390x.",
             ),
             (
                 "s390x",
@@ -61,12 +61,10 @@ class TestCommonCriteriaEntitlementUserFacingStatus:
             ),
         ),
     )
-    @mock.patch(M_REPOPATH + "os.getuid", return_value=0)
     @mock.patch("uaclient.system.get_platform_info")
     def test_inapplicable_on_invalid_affordances(
         self,
         m_platform_info,
-        m_getuid,
         arch,
         series,
         version,
@@ -166,9 +164,7 @@ class TestCommonCriteriaEntitlementEnable:
 
         with mock.patch("uaclient.apt.add_auth_apt_repo") as m_add_apt:
             with mock.patch("uaclient.apt.add_ppa_pinning") as m_add_pin:
-                with mock.patch(
-                    M_REPOPATH + "os.path.exists", side_effect=exists
-                ):
+                with mock.patch(M_REPOPATH + "exists", side_effect=exists):
                     assert (True, None) == entitlement.enable()
 
         add_apt_calls = [
