@@ -10,6 +10,7 @@ from functools import partial
 import mock
 import pytest
 
+import uaclient.entitlements.fips as fips
 from uaclient import apt, defaults, exceptions, messages, system, util
 from uaclient.clouds.identity import NoCloudTypeReason
 from uaclient.entitlements.entitlement_status import (
@@ -26,6 +27,7 @@ from uaclient.entitlements.fips import (
     FIPSEntitlement,
     FIPSUpdatesEntitlement,
 )
+from uaclient.files.notices import Notice, NoticesManager
 
 M_PATH = "uaclient.entitlements.fips."
 M_LIVEPATCH_PATH = "uaclient.entitlements.livepatch.LivepatchEntitlement."
@@ -263,9 +265,13 @@ class TestFIPSEntitlementEnable:
     @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "get_cloud_type", return_value=("", None))
     def test_enable_configures_apt_sources_and_auth_files(
-        self, _m_get_cloud_type, m_setup_apt_proxy, entitlement
+        self,
+        _m_get_cloud_type,
+        m_setup_apt_proxy,
+        entitlement,
     ):
         """When entitled, configure apt repo auth token, pinning and url."""
+        notice_ent_cls = NoticesManager()
         patched_packages = ["a", "b"]
         expected_conditional_packages = [
             "openssh-server",
@@ -283,7 +289,7 @@ class TestFIPSEntitlementEnable:
             )
             m_installed_pkgs = stack.enter_context(
                 mock.patch(
-                    "uaclient.apt.get_installed_packages",
+                    "uaclient.apt.get_installed_packages_names",
                     return_value=["openssh-server", "strongswan"],
                 )
             )
@@ -299,7 +305,16 @@ class TestFIPSEntitlementEnable:
             stack.enter_context(
                 mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
             )
-            stack.enter_context(mock.patch(M_REPOPATH + "os.path.exists"))
+            stack.enter_context(
+                mock.patch(
+                    "uaclient.entitlements.fips.system.should_reboot",
+                    return_value=True,
+                )
+            )
+            stack.enter_context(mock.patch(M_REPOPATH + "exists"))
+            stack.enter_context(
+                mock.patch.object(fips, "services_once_enabled_file")
+            )
             # Note that this patch uses a PropertyMock and happens on the
             # entitlement's type because packages is a property
             m_packages = mock.PropertyMock(return_value=patched_packages)
@@ -393,20 +408,20 @@ class TestFIPSEntitlementEnable:
         assert apt_pinning_calls == m_add_pinning.call_args_list
         assert subp_calls == m_subp.call_args_list
         assert [
-            ["", messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg]
-        ] == entitlement.cfg.notice_file.read()
+            messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+        ] == notice_ent_cls.list()
 
     @pytest.mark.parametrize(
         "fips_common_enable_return_value, expected_remove_notice_calls",
         [
-            (True, [mock.call("", messages.FIPS_INSTALL_OUT_OF_DATE)]),
+            (True, [mock.call(Notice.FIPS_INSTALL_OUT_OF_DATE)]),
             (False, []),
         ],
     )
     @mock.patch(
         "uaclient.entitlements.fips.FIPSCommonEntitlement._perform_enable"
     )
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     def test_enable_removes_out_of_date_notice_on_success(
         self,
         m_remove_notice,
@@ -430,16 +445,18 @@ class TestFIPSEntitlementEnable:
                 True,
                 [
                     mock.call(
-                        "", messages.NOTICE_WRONG_FIPS_METAPACKAGE_ON_CLOUD
+                        Notice.WRONG_FIPS_METAPACKAGE_ON_CLOUD,
+                    ),
+                    mock.call(
+                        Notice.FIPS_REBOOT_REQUIRED,
                     ),
-                    mock.call("", messages.FIPS_REBOOT_REQUIRED_MSG),
                 ],
             ),
             (False, []),
         ],
     )
     @mock.patch("uaclient.entitlements.repo.RepoEntitlement._perform_enable")
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     def test_enable_removes_wrong_met_notice_on_success(
         self,
         m_remove_notice,
@@ -449,7 +466,8 @@ class TestFIPSEntitlementEnable:
         entitlement,
     ):
         m_repo_enable.return_value = repo_enable_return_value
-        assert repo_enable_return_value is entitlement._perform_enable()
+        with mock.patch.object(fips, "services_once_enabled_file"):
+            assert repo_enable_return_value is entitlement._perform_enable()
         assert (
             expected_remove_notice_calls == m_remove_notice.call_args_list[:2]
         )
@@ -509,7 +527,7 @@ class TestFIPSEntitlementEnable:
             stack.enter_context(
                 mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
             )
-            stack.enter_context(mock.patch(M_REPOPATH + "os.path.exists"))
+            stack.enter_context(mock.patch(M_REPOPATH + "exists"))
 
             with pytest.raises(exceptions.UserFacingError) as excinfo:
                 entitlement.enable()
@@ -523,53 +541,6 @@ class TestFIPSEntitlementEnable:
         assert 0 == m_add_pinning.call_count
         assert 0 == m_remove_apt_config.call_count
 
-    @mock.patch("uaclient.apt.setup_apt_proxy")
-    def test_failure_to_install_removes_apt_auth(
-        self, _m_setup_apt_proxy, entitlement, tmpdir
-    ):
-
-        authfile = tmpdir.join("90ubuntu-advantage")
-        authfile.write("")
-
-        def fake_subp(cmd, *args, **kwargs):
-            if "install" in cmd:
-                raise exceptions.ProcessExecutionError(cmd)
-            return ("", "")
-
-        with contextlib.ExitStack() as stack:
-            stack.enter_context(
-                mock.patch("uaclient.system.subp", side_effect=fake_subp)
-            )
-            stack.enter_context(
-                mock.patch.object(
-                    entitlement, "can_enable", return_value=(True, None)
-                )
-            )
-            stack.enter_context(
-                mock.patch("uaclient.util.handle_message_operations")
-            )
-            stack.enter_context(
-                mock.patch.object(
-                    entitlement, "setup_apt_config", return_value=True
-                )
-            )
-            m_remove_apt_config = stack.enter_context(
-                mock.patch.object(
-                    entitlement, "remove_apt_config", return_value=True
-                )
-            )
-            stack.enter_context(
-                mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
-            )
-            stack.enter_context(mock.patch(M_REPOPATH + "os.path.exists"))
-
-            with pytest.raises(exceptions.UserFacingError) as excinfo:
-                entitlement.enable()
-            error_msg = "Could not enable {}.".format(entitlement.title)
-            assert error_msg == excinfo.value.msg
-
-        assert 1 == m_remove_apt_config.call_count
-
     @mock.patch(
         "uaclient.entitlements.fips.get_cloud_type", return_value=("", None)
     )
@@ -660,20 +631,25 @@ class TestFIPSEntitlementEnable:
         entitlement_factory,
     ):
         m_handle_message_op.return_value = True
-        fips_entitlement = entitlement_factory(
-            FIPSEntitlement, services_once_enabled={"fips-updates": True}
-        )
+        fake_dof = mock.MagicMock()
+        fake_ua_file = mock.MagicMock()
+        fake_ua_file.to_json.return_value = {"fips_updates": True}
+        fake_dof.read.return_value = fake_ua_file
+        fips_entitlement = entitlement_factory(FIPSEntitlement)
 
         with mock.patch.object(
             fips_entitlement, "_allow_fips_on_cloud_instance"
         ) as m_allow_fips_on_cloud:
             m_allow_fips_on_cloud.return_value = True
-            result, reason = fips_entitlement.enable()
-            assert not result
-            expected_msg = (
-                "Cannot enable FIPS because FIPS Updates was once enabled."
-            )
-            assert expected_msg.strip() == reason.message.msg.strip()
+            with mock.patch.object(
+                fips, "services_once_enabled_file", fake_dof
+            ):
+                result, reason = fips_entitlement.enable()
+                assert not result
+                expected_msg = (
+                    "Cannot enable FIPS because FIPS Updates was once enabled."
+                )
+                assert expected_msg.strip() == reason.message.msg.strip()
 
     @mock.patch("uaclient.system.get_platform_info")
     @mock.patch("uaclient.entitlements.fips.get_cloud_type")
@@ -843,7 +819,7 @@ class TestFIPSEntitlementRemovePackages:
     @pytest.mark.parametrize("installed_pkgs", (["sl"], ["ubuntu-fips", "sl"]))
     @mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
     @mock.patch(M_PATH + "system.subp")
-    @mock.patch(M_PATH + "apt.get_installed_packages")
+    @mock.patch(M_PATH + "apt.get_installed_packages_names")
     def test_remove_packages_only_removes_if_package_is_installed(
         self,
         m_get_installed_packages,
@@ -875,7 +851,7 @@ class TestFIPSEntitlementRemovePackages:
 
     @mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
     @mock.patch(M_PATH + "system.subp")
-    @mock.patch(M_PATH + "apt.get_installed_packages")
+    @mock.patch(M_PATH + "apt.get_installed_packages_names")
     def test_remove_packages_output_message_when_fail(
         self, m_get_installed_packages, m_subp, _m_get_platform, entitlement
     ):
@@ -903,6 +879,8 @@ class TestFIPSEntitlementDisable:
         entitlement,
     ):
         """When can_disable, disable removes apt config and packages."""
+        notice_ent_cls = NoticesManager()
+
         with mock.patch.object(
             entitlement, "can_disable", return_value=(True, None)
         ):
@@ -916,8 +894,8 @@ class TestFIPSEntitlementDisable:
         assert [mock.call(silent=True)] == m_remove_apt_config.call_args_list
         assert [mock.call()] == m_remove_packages.call_args_list
         assert [
-            ["", messages.FIPS_DISABLE_REBOOT_REQUIRED]
-        ] == entitlement.cfg.notice_file.read()
+            messages.FIPS_DISABLE_REBOOT_REQUIRED,
+        ] == notice_ent_cls.list()
 
 
 @mock.patch("uaclient.system.should_reboot")
@@ -945,7 +923,7 @@ class TestFIPSEntitlementApplicationStatus:
     def test_non_root_does_not_fail(
         self, _m_should_reboot, super_application_status, FakeConfig
     ):
-        cfg = FakeConfig(root_mode=False)
+        cfg = FakeConfig()
         entitlement = FIPSUpdatesEntitlement(cfg)
         msg = "sure is some status here"
         with mock.patch(
@@ -967,6 +945,7 @@ class TestFIPSEntitlementApplicationStatus:
         should_reboot,
         entitlement,
     ):
+        notice_ent_cls = NoticesManager()
         m_should_reboot.return_value = should_reboot
         orig_load_file = system.load_file
 
@@ -983,18 +962,21 @@ class TestFIPSEntitlementApplicationStatus:
             return orig_exists(path)
 
         msg = messages.NamedMessage("test-code", "sure is some status here")
-        entitlement.cfg.notice_file.add(
-            "", messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg
+        notice_ent_cls.add(
+            Notice.FIPS_SYSTEM_REBOOT_REQUIRED,
+            messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
         )
 
         if path_exists:
-            entitlement.cfg.notice_file.add(
-                "", messages.FIPS_REBOOT_REQUIRED_MSG
+            notice_ent_cls.add(
+                Notice.FIPS_REBOOT_REQUIRED,
+                messages.FIPS_REBOOT_REQUIRED_MSG,
             )
 
         if proc_content == "0":
-            entitlement.cfg.notice_file.add(
-                "", messages.FIPS_DISABLE_REBOOT_REQUIRED
+            notice_ent_cls.add(
+                Notice.FIPS_DISABLE_REBOOT_REQUIRED,
+                messages.FIPS_DISABLE_REBOOT_REQUIRED,
             )
 
         with mock.patch(
@@ -1015,28 +997,37 @@ class TestFIPSEntitlementApplicationStatus:
         if path_exists and should_reboot and proc_content == "1":
             expected_msg = msg
             assert [
-                ["", messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg]
-            ] == entitlement.cfg.notice_file.read()
+                messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+                messages.FIPS_REBOOT_REQUIRED_MSG,
+            ] == notice_ent_cls.list()
         elif path_exists and not should_reboot and proc_content == "1":
             expected_msg = msg
-            assert entitlement.cfg.notice_file.read() is None
+            # we do not delete the FIPS_REBOOT_REQUIRED notices
+            # deleting will happen after rebooting
+            assert notice_ent_cls.list() == [messages.FIPS_REBOOT_REQUIRED_MSG]
         elif path_exists and should_reboot and proc_content == "0":
             expected_msg = messages.FIPS_PROC_FILE_ERROR.format(
                 file_name=entitlement.FIPS_PROC_FILE
             )
             expected_status = ApplicationStatus.DISABLED
             assert [
-                ["", messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg],
-                ["", messages.NOTICE_FIPS_MANUAL_DISABLE_URL],
-            ] == entitlement.cfg.notice_file.read()
+                messages.FIPS_DISABLE_REBOOT_REQUIRED,
+                messages.NOTICE_FIPS_MANUAL_DISABLE_URL,
+                messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg,
+                messages.FIPS_REBOOT_REQUIRED_MSG,
+            ] == notice_ent_cls.list()
         elif path_exists and not should_reboot and proc_content == "0":
             expected_msg = messages.FIPS_PROC_FILE_ERROR.format(
                 file_name=entitlement.FIPS_PROC_FILE
             )
             expected_status = ApplicationStatus.DISABLED
             assert [
-                ["", messages.NOTICE_FIPS_MANUAL_DISABLE_URL],
-            ] == entitlement.cfg.notice_file.read()
+                (
+                    Notice.FIPS_MANUAL_DISABLE_URL.value.label,
+                    messages.NOTICE_FIPS_MANUAL_DISABLE_URL,
+                )
+                == notice_ent_cls.list()
+            ]
         else:
             expected_msg = messages.FIPS_REBOOT_REQUIRED
 
@@ -1069,11 +1060,11 @@ class TestFipsEntitlementInstallPackages:
         self, m_run_apt, entitlement
     ):
         m_run_apt.side_effect = exceptions.UserFacingError("error")
-        with mock.patch.object(entitlement, "_cleanup"):
+        with mock.patch.object(entitlement, "remove_apt_config"):
             with pytest.raises(exceptions.UserFacingError):
                 entitlement.install_packages()
 
-    @mock.patch(M_PATH + "apt.get_installed_packages")
+    @mock.patch(M_PATH + "apt.get_installed_packages_names")
     @mock.patch(M_PATH + "apt.run_apt_install_command")
     def test_install_packages_dont_fail_if_conditional_pkgs_not_installed(
         self,
@@ -1173,7 +1164,7 @@ class TestFipsSetupAPTConfig:
 
 
 class TestFipsEntitlementPackages:
-    @mock.patch(M_PATH + "apt.get_installed_packages", return_value=[])
+    @mock.patch(M_PATH + "apt.get_installed_packages_names", return_value=[])
     @mock.patch("uaclient.system.get_platform_info")
     def test_packages_is_list(self, m_platform_info, _mock, entitlement):
         """RepoEntitlement.enable will fail if it isn't"""
@@ -1184,7 +1175,7 @@ class TestFipsEntitlementPackages:
 
         assert isinstance(entitlement.packages, list)
 
-    @mock.patch(M_PATH + "apt.get_installed_packages", return_value=[])
+    @mock.patch(M_PATH + "apt.get_installed_packages_names", return_value=[])
     @mock.patch("uaclient.system.get_platform_info")
     def test_fips_required_packages_included(
         self, m_platform_info, _mock, entitlement
@@ -1210,7 +1201,7 @@ class TestFipsEntitlementPackages:
 
         assert sorted(FIPS_ADDITIONAL_PACKAGES) == sorted(entitlement.packages)
 
-    @mock.patch(M_PATH + "apt.get_installed_packages")
+    @mock.patch(M_PATH + "apt.get_installed_packages_names")
     @mock.patch("uaclient.system.get_platform_info")
     def test_multiple_packages_calls_dont_mutate_state(
         self, m_platform_info, m_get_installed_packages, entitlement
@@ -1230,81 +1221,10 @@ class TestFipsEntitlementPackages:
 
         assert before == after
 
-    @pytest.mark.parametrize(
-        "cfg_disable_fips_metapckage_override", (True, False)
-    )
-    @pytest.mark.parametrize("series", ("xenial", "bionic", "focal"))
-    @pytest.mark.parametrize(
-        "cloud_id",
-        (
-            "azure-china",
-            "aws-gov",
-            "aws-china",
-            "azure",
-            "aws",
-            "gce",
-            None,
-        ),
-    )
-    @mock.patch("uaclient.util.is_config_value_true")
-    @mock.patch(M_PATH + "get_cloud_type")
-    @mock.patch("uaclient.system.get_platform_info")
-    @mock.patch("uaclient.apt.get_installed_packages")
-    def test_packages_are_override_on_cloud_instance(
-        self,
-        m_installed_packages,
-        m_platform_info,
-        m_get_cloud_type,
-        m_is_config_value,
-        cloud_id,
-        series,
-        cfg_disable_fips_metapckage_override,
-        fips_entitlement_factory,
-    ):
-        m_platform_info.return_value = {"series": series}
-        m_get_cloud_type.return_value = (
-            (cloud_id, None)
-            if cloud_id is not None
-            else (None, NoCloudTypeReason.NO_CLOUD_DETECTED)
-        )
-        m_installed_packages.return_value = []
-        m_is_config_value.return_value = cfg_disable_fips_metapckage_override
-        additional_packages = ["test1", "ubuntu-fips", "test2"]
-        entitlement = fips_entitlement_factory(
-            additional_packages=additional_packages
-        )
-
-        packages = entitlement.packages
-
-        if all(
-            [
-                series in ("bionic", "focal"),
-                cloud_id
-                in (
-                    "azure",
-                    "aws",
-                    "aws-china",
-                    "aws-gov",
-                    "azure-china",
-                    "gce",
-                ),
-                not cfg_disable_fips_metapckage_override,
-            ]
-        ):
-            cloud_id = cloud_id.split("-")[0]
-            cloud_id = "gcp" if cloud_id == "gce" else cloud_id
-            assert packages == [
-                "test1",
-                "ubuntu-{}-fips".format(cloud_id),
-                "test2",
-            ]
-        else:
-            assert packages == additional_packages
-
 
 class TestFIPSUpdatesEntitlementEnable:
     @pytest.mark.parametrize("enable_ret", ((True), (False)))
-    @mock.patch("uaclient.files.NoticeFile.try_remove")
+    @mock.patch("uaclient.entitlements.fips.notices.NoticesManager.remove")
     @mock.patch(
         "uaclient.entitlements.fips.FIPSCommonEntitlement._perform_enable"
     )
@@ -1316,32 +1236,22 @@ class TestFIPSUpdatesEntitlementEnable:
         entitlement_factory,
     ):
         m_perform_enable.return_value = enable_ret
-        m_write_cache = mock.MagicMock()
-        m_read_cache = mock.MagicMock()
-        m_read_cache.return_value = {}
+        fake_file = mock.MagicMock()
+        fake_file.read.return_value = None
 
-        cfg = mock.MagicMock()
-        cfg.read_cache = m_read_cache
-        cfg.write_cache = m_write_cache
-        cfg.notice_file.try_remove = m_remove_notice
-
-        fips_updates_ent = entitlement_factory(FIPSUpdatesEntitlement, cfg=cfg)
-        assert fips_updates_ent._perform_enable() == enable_ret
+        with mock.patch.object(
+            fips, "services_once_enabled_file", fake_file
+        ) as m_services_once_enabled:
+            cfg = mock.MagicMock()
+            fips_updates_ent = entitlement_factory(
+                FIPSUpdatesEntitlement, cfg=cfg
+            )
+            assert fips_updates_ent._perform_enable() == enable_ret
 
         if enable_ret:
-            assert 1 == m_read_cache.call_count
-            assert 1 == m_write_cache.call_count
-            assert [
-                mock.call(
-                    key="services-once-enabled", content={"fips-updates": True}
-                )
-            ] == m_write_cache.call_args_list
-            assert [
-                mock.call("", messages.FIPS_DISABLE_REBOOT_REQUIRED)
-            ] == m_remove_notice.call_args_list
+            assert 1 == m_services_once_enabled.write.call_count
         else:
-            assert not m_read_cache.call_count
-            assert not m_write_cache.call_count
+            assert not m_services_once_enabled.write.call_count
 
 
 class TestFIPSUpdatesEntitlementCanEnable:
