@@ -1,4 +1,4 @@
-"""Tests related to uaclient.entitlement.base module."""
+"""Tests related to uaclient.entitlement.livepatch module."""
 
 import contextlib
 import copy
@@ -10,7 +10,7 @@ from types import MappingProxyType
 import mock
 import pytest
 
-from uaclient import apt, exceptions, messages, system
+from uaclient import apt, exceptions, livepatch, messages, system
 from uaclient.entitlements.entitlement_status import (
     ApplicabilityStatus,
     ApplicationStatus,
@@ -19,12 +19,8 @@ from uaclient.entitlements.entitlement_status import (
     UserFacingStatus,
 )
 from uaclient.entitlements.livepatch import (
-    LIVEPATCH_CMD,
     LivepatchEntitlement,
-    configure_livepatch_proxy,
-    get_config_option_value,
     process_config_directives,
-    unconfigure_livepatch_proxy,
 )
 from uaclient.entitlements.tests.conftest import machine_token
 from uaclient.snap import SNAP_CMD
@@ -68,166 +64,6 @@ def entitlement(livepatch_entitlement_factory):
     return livepatch_entitlement_factory()
 
 
-class TestConfigureLivepatchProxy:
-    @pytest.mark.parametrize(
-        "http_proxy,https_proxy,retry_sleeps",
-        (
-            ("http_proxy", "https_proxy", [1, 2]),
-            ("http_proxy", "", None),
-            ("", "https_proxy", [1, 2]),
-            ("http_proxy", None, [1, 2]),
-            (None, "https_proxy", None),
-            (None, None, [1, 2]),
-        ),
-    )
-    @mock.patch("uaclient.system.subp")
-    def test_configure_livepatch_proxy(
-        self, m_subp, http_proxy, https_proxy, retry_sleeps, capsys, event
-    ):
-        configure_livepatch_proxy(http_proxy, https_proxy, retry_sleeps)
-        expected_calls = []
-        if http_proxy:
-            expected_calls.append(
-                mock.call(
-                    [
-                        LIVEPATCH_CMD,
-                        "config",
-                        "http-proxy={}".format(http_proxy),
-                    ],
-                    retry_sleeps=retry_sleeps,
-                )
-            )
-
-        if https_proxy:
-            expected_calls.append(
-                mock.call(
-                    [
-                        LIVEPATCH_CMD,
-                        "config",
-                        "https-proxy={}".format(https_proxy),
-                    ],
-                    retry_sleeps=retry_sleeps,
-                )
-            )
-
-        assert m_subp.call_args_list == expected_calls
-
-        out, _ = capsys.readouterr()
-        if http_proxy or https_proxy:
-            assert out.strip() == messages.SETTING_SERVICE_PROXY.format(
-                service=LivepatchEntitlement.title
-            )
-
-    @pytest.mark.parametrize(
-        "key, subp_return_value, expected_ret",
-        [
-            ("http-proxy", ("nonsense", ""), None),
-            ("http-proxy", ("", "nonsense"), None),
-            (
-                "http-proxy",
-                (
-                    """\
-http-proxy: ""
-https-proxy: ""
-no-proxy: ""
-remote-server: https://livepatch.canonical.com
-ca-certs: ""
-check-interval: 60  # minutes""",
-                    "",
-                ),
-                None,
-            ),
-            (
-                "http-proxy",
-                (
-                    """\
-http-proxy: one
-https-proxy: two
-no-proxy: ""
-remote-server: https://livepatch.canonical.com
-ca-certs: ""
-check-interval: 60  # minutes""",
-                    "",
-                ),
-                "one",
-            ),
-            (
-                "https-proxy",
-                (
-                    """\
-http-proxy: one
-https-proxy: two
-no-proxy: ""
-remote-server: https://livepatch.canonical.com
-ca-certs: ""
-check-interval: 60  # minutes""",
-                    "",
-                ),
-                "two",
-            ),
-            (
-                "nonexistentkey",
-                (
-                    """\
-http-proxy: one
-https-proxy: two
-no-proxy: ""
-remote-server: https://livepatch.canonical.com
-ca-certs: ""
-check-interval: 60  # minutes""",
-                    "",
-                ),
-                None,
-            ),
-        ],
-    )
-    @mock.patch("uaclient.system.subp")
-    def test_get_config_option_value(
-        self, m_util_subp, key, subp_return_value, expected_ret
-    ):
-        m_util_subp.return_value = subp_return_value
-        ret = get_config_option_value(key)
-        assert ret == expected_ret
-        assert [
-            mock.call([LIVEPATCH_CMD, "config"])
-        ] == m_util_subp.call_args_list
-
-
-class TestUnconfigureLivepatchProxy:
-    @pytest.mark.parametrize(
-        "livepatch_installed, protocol_type, retry_sleeps",
-        (
-            (True, "http", None),
-            (True, "https", [1]),
-            (True, "http", []),
-            (False, "http", None),
-        ),
-    )
-    @mock.patch("uaclient.system.which")
-    @mock.patch("uaclient.system.subp")
-    def test_unconfigure_livepatch_proxy(
-        self, subp, which, livepatch_installed, protocol_type, retry_sleeps
-    ):
-        if livepatch_installed:
-            which.return_value = LIVEPATCH_CMD
-        else:
-            which.return_value = None
-        kwargs = {"protocol_type": protocol_type}
-        if retry_sleeps is not None:
-            kwargs["retry_sleeps"] = retry_sleeps
-        assert None is unconfigure_livepatch_proxy(**kwargs)
-        if livepatch_installed:
-            expected_calls = [
-                mock.call(
-                    [LIVEPATCH_CMD, "config", protocol_type + "-proxy="],
-                    retry_sleeps=retry_sleeps,
-                )
-            ]
-        else:
-            expected_calls = []
-        assert expected_calls == subp.call_args_list
-
-
 class TestLivepatchContractStatus:
     def test_contract_status_entitled(self, entitlement):
         """The contract_status returns ENTITLED when entitled is True."""
@@ -240,12 +76,19 @@ class TestLivepatchContractStatus:
 
 
 class TestLivepatchUserFacingStatus:
+    @mock.patch(
+        "uaclient.livepatch.on_supported_kernel",
+        return_value=None,
+    )
     @mock.patch(
         "uaclient.entitlements.livepatch.system.is_container",
-        return_value=False,
+        return_value=True,
     )
     def test_user_facing_status_inapplicable_on_inapplicable_status(
-        self, _m_is_container, livepatch_entitlement_factory
+        self,
+        _m_is_container,
+        _m_on_supported_kernel,
+        livepatch_entitlement_factory,
     ):
         """The user-facing details INAPPLICABLE applicability_status"""
         affordances = copy.deepcopy(DEFAULT_AFFORDANCES)
@@ -259,10 +102,7 @@ class TestLivepatchUserFacingStatus:
             m_platform_info.return_value = PLATFORM_INFO_SUPPORTED
             uf_status, details = entitlement.user_facing_status()
         assert uf_status == UserFacingStatus.INAPPLICABLE
-        expected_details = (
-            "Livepatch is not available for Ubuntu 16.04 LTS"
-            " (Xenial Xerus)."
-        )
+        expected_details = "Cannot install Livepatch on a container."
         assert expected_details == details.msg
 
     def test_user_facing_status_unavailable_on_unentitled(self, entitlement):
@@ -298,7 +138,7 @@ class TestLivepatchProcessConfigDirectives:
             process_config_directives(cfg)
         expected_subp = mock.call(
             [
-                LIVEPATCH_CMD,
+                livepatch.LIVEPATCH_CMD,
                 "config",
                 livepatch_param_tmpl.format(directive_value),
             ],
@@ -321,10 +161,12 @@ class TestLivepatchProcessConfigDirectives:
             process_config_directives(cfg)
         expected_calls = [
             mock.call(
-                [LIVEPATCH_CMD, "config", "ca-certs=value2"], capture=True
+                [livepatch.LIVEPATCH_CMD, "config", "ca-certs=value2"],
+                capture=True,
             ),
             mock.call(
-                [LIVEPATCH_CMD, "config", "remote-server=value1"], capture=True
+                [livepatch.LIVEPATCH_CMD, "config", "remote-server=value1"],
+                capture=True,
             ),
         ]
         assert expected_calls == m_subp.call_args_list
@@ -403,169 +245,6 @@ class TestLivepatchEntitlementCanEnable:
         assert ("", "") == capsys.readouterr()
         assert [mock.call()] == m_container.call_args_list
 
-    @mock.patch(
-        "uaclient.system.get_kernel_info",
-        return_value=system.KernelInfo(
-            uname_release="4.4.0-140-notgeneric",
-            proc_version_signature_version="",
-            major=4,
-            minor=4,
-            patch=0,
-            abi="140",
-            flavor="notgeneric",
-        ),
-    )
-    @mock.patch(
-        "uaclient.system.get_platform_info",
-        return_value=PLATFORM_INFO_SUPPORTED,
-    )
-    def test_can_enable_false_on_unsupported_kernel_flavor(
-        self,
-        _m_platform,
-        _m_kernel_info,
-        _m_is_container,
-        _m_livepatch_status,
-        _m_fips_status,
-        entitlement,
-    ):
-        """When on an unsupported kernel, can_enable returns False."""
-        entitlement = LivepatchEntitlement(entitlement.cfg)
-        result, reason = entitlement.can_enable()
-        assert False is result
-        assert CanEnableFailureReason.INAPPLICABLE == reason.reason
-        msg = (
-            "Livepatch is not available for kernel 4.4.0-140-notgeneric.\n"
-            "Supported flavors are: generic, lowlatency."
-        )
-        assert msg == reason.message.msg
-
-    @pytest.mark.parametrize(
-        "kernel_info,meets_min_version",
-        (
-            (
-                system.KernelInfo(
-                    uname_release="3.5.0-00-generic",
-                    proc_version_signature_version="",
-                    major=3,
-                    minor=5,
-                    patch=0,
-                    abi="00",
-                    flavor="generic",
-                ),
-                False,
-            ),
-            (
-                system.KernelInfo(
-                    uname_release="4.2.9-00-generic",
-                    proc_version_signature_version="",
-                    major=4,
-                    minor=2,
-                    patch=9,
-                    abi="00",
-                    flavor="generic",
-                ),
-                False,
-            ),
-            (
-                system.KernelInfo(
-                    uname_release="4.3.0-00-generic",
-                    proc_version_signature_version="",
-                    major=4,
-                    minor=3,
-                    patch=0,
-                    abi="00",
-                    flavor="generic",
-                ),
-                False,
-            ),
-            (
-                system.KernelInfo(
-                    uname_release="4.4.0-00-generic",
-                    proc_version_signature_version="",
-                    major=4,
-                    minor=4,
-                    patch=0,
-                    abi="00",
-                    flavor="generic",
-                ),
-                True,
-            ),
-            (
-                system.KernelInfo(
-                    uname_release="4.10.0-00-generic",
-                    proc_version_signature_version="",
-                    major=4,
-                    minor=10,
-                    patch=0,
-                    abi="00",
-                    flavor="generic",
-                ),
-                True,
-            ),
-            (
-                system.KernelInfo(
-                    uname_release="5.0.0-00-generic",
-                    proc_version_signature_version="",
-                    major=5,
-                    minor=0,
-                    patch=0,
-                    abi="00",
-                    flavor="generic",
-                ),
-                True,
-            ),
-        ),
-    )
-    @mock.patch("uaclient.system.get_kernel_info")
-    @mock.patch(
-        "uaclient.system.get_platform_info",
-        return_value=PLATFORM_INFO_SUPPORTED,
-    )
-    def test_can_enable_false_on_unsupported_min_kernel_version(
-        self,
-        _m_platform,
-        m_kernel_info,
-        _m_is_container,
-        _m_livepatch_status,
-        _m_fips_status,
-        kernel_info,
-        meets_min_version,
-        entitlement,
-    ):
-        """When on an unsupported kernel version, can_enable returns False."""
-        m_kernel_info.return_value = kernel_info
-        entitlement = LivepatchEntitlement(entitlement.cfg)
-        if meets_min_version:
-            assert (True, None) == entitlement.can_enable()
-        else:
-            result, reason = entitlement.can_enable()
-            assert False is result
-            assert CanEnableFailureReason.INAPPLICABLE == reason.reason
-            msg = (
-                "Livepatch is not available for kernel {}.\n"
-                "Minimum kernel version required: 4.4.".format(
-                    kernel_info.uname_release
-                )
-            )
-            assert msg == reason.message.msg
-
-    def test_can_enable_false_on_unsupported_architecture(
-        self, _m_is_container, _m_livepatch_status, _m_fips_status, entitlement
-    ):
-        """When on an unsupported architecture, can_enable returns False."""
-        unsupported_kernel = copy.deepcopy(dict(PLATFORM_INFO_SUPPORTED))
-        unsupported_kernel["arch"] = "ppc64le"
-        with mock.patch("uaclient.system.get_platform_info") as m_platform:
-            m_platform.return_value = unsupported_kernel
-            result, reason = entitlement.can_enable()
-            assert False is result
-            assert CanEnableFailureReason.INAPPLICABLE == reason.reason
-            msg = (
-                "Livepatch is not available for platform ppc64le.\n"
-                "Supported platforms are: x86_64."
-            )
-            assert msg == reason.message.msg
-
     def test_can_enable_false_on_containers(
         self, m_is_container, _m_livepatch_status, _m_fips_status, entitlement
     ):
@@ -687,14 +366,13 @@ class TestLivepatchProcessContractDeltas:
 @mock.patch(M_PATH + "snap.is_installed")
 @mock.patch("uaclient.util.validate_proxy", side_effect=lambda x, y, z: y)
 @mock.patch("uaclient.snap.configure_snap_proxy")
-@mock.patch("uaclient.entitlements.livepatch.configure_livepatch_proxy")
+@mock.patch("uaclient.livepatch.configure_livepatch_proxy")
 class TestLivepatchEntitlementEnable:
 
     mocks_apt_update = [mock.call()]
     mocks_snapd_install = [
         mock.call(
             ["apt-get", "install", "--assume-yes", "snapd"],
-            capture=True,
             retry_sleeps=apt.APT_RETRIES,
         )
     ]
@@ -716,14 +394,17 @@ class TestLivepatchEntitlementEnable:
     mocks_config = [
         mock.call(
             [
-                LIVEPATCH_CMD,
+                livepatch.LIVEPATCH_CMD,
                 "config",
                 "remote-server=https://alt.livepatch.com",
             ],
             capture=True,
         ),
-        mock.call([LIVEPATCH_CMD, "disable"]),
-        mock.call([LIVEPATCH_CMD, "enable", "livepatch-token"], capture=True),
+        mock.call([livepatch.LIVEPATCH_CMD, "disable"]),
+        mock.call(
+            [livepatch.LIVEPATCH_CMD, "enable", "livepatch-token"],
+            capture=True,
+        ),
     ]
 
     @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
@@ -733,7 +414,7 @@ class TestLivepatchEntitlementEnable:
     @mock.patch("uaclient.contract.apply_contract_overrides")
     @mock.patch("uaclient.apt.run_apt_install_command")
     @mock.patch("uaclient.apt.run_apt_update_command")
-    @mock.patch("uaclient.system.which", return_value=False)
+    @mock.patch("uaclient.system.which", return_value=None)
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
     @mock.patch(
         M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
@@ -788,7 +469,10 @@ class TestLivepatchEntitlementEnable:
             assert expected_log not in caplog_text()
         else:
             assert expected_log in caplog_text()
-        expected_calls = [mock.call("/usr/bin/snap"), mock.call(LIVEPATCH_CMD)]
+        expected_calls = [
+            mock.call("/usr/bin/snap"),
+            mock.call(livepatch.LIVEPATCH_CMD),
+        ]
         assert expected_calls == m_which.call_args_list
         assert m_validate_proxy.call_count == 2
         assert m_snap_proxy.call_count == 1
@@ -798,7 +482,8 @@ class TestLivepatchEntitlementEnable:
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.contract.apply_contract_overrides")
     @mock.patch(
-        "uaclient.system.which", side_effect=lambda cmd: cmd == "/usr/bin/snap"
+        "uaclient.system.which",
+        side_effect=lambda cmd: cmd if cmd == "/usr/bin/snap" else None,
     )
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
     @mock.patch(
@@ -837,7 +522,10 @@ class TestLivepatchEntitlementEnable:
             "Canonical livepatch enabled.\n"
         )
         assert (msg, "") == capsys.readouterr()
-        expected_calls = [mock.call("/usr/bin/snap"), mock.call(LIVEPATCH_CMD)]
+        expected_calls = [
+            mock.call("/usr/bin/snap"),
+            mock.call(livepatch.LIVEPATCH_CMD),
+        ]
         assert expected_calls == m_which.call_args_list
         assert m_validate_proxy.call_count == 2
         assert m_snap_proxy.call_count == 1
@@ -845,7 +533,8 @@ class TestLivepatchEntitlementEnable:
 
     @mock.patch("uaclient.system.subp")
     @mock.patch(
-        "uaclient.system.which", side_effect=lambda cmd: cmd == "/usr/bin/snap"
+        "uaclient.system.which",
+        side_effect=lambda cmd: cmd if cmd == "/usr/bin/snap" else None,
     )
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
     @mock.patch(
@@ -883,7 +572,9 @@ class TestLivepatchEntitlementEnable:
     @mock.patch("uaclient.system.get_platform_info")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.contract.apply_contract_overrides")
-    @mock.patch("uaclient.system.which", side_effect=[True, True])
+    @mock.patch(
+        "uaclient.system.which", side_effect=["/path/to/exe", "/path/to/exe"]
+    )
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
     @mock.patch(
         M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
@@ -916,15 +607,16 @@ class TestLivepatchEntitlementEnable:
             ),
             mock.call(
                 [
-                    LIVEPATCH_CMD,
+                    livepatch.LIVEPATCH_CMD,
                     "config",
                     "remote-server=https://alt.livepatch.com",
                 ],
                 capture=True,
             ),
-            mock.call([LIVEPATCH_CMD, "disable"]),
+            mock.call([livepatch.LIVEPATCH_CMD, "disable"]),
             mock.call(
-                [LIVEPATCH_CMD, "enable", "livepatch-token"], capture=True
+                [livepatch.LIVEPATCH_CMD, "enable", "livepatch-token"],
+                capture=True,
             ),
         ]
         assert subp_calls == m_subp.call_args_list
@@ -936,7 +628,9 @@ class TestLivepatchEntitlementEnable:
     @mock.patch("uaclient.system.get_platform_info")
     @mock.patch("uaclient.system.subp")
     @mock.patch("uaclient.contract.apply_contract_overrides")
-    @mock.patch("uaclient.system.which", side_effect=[True, True])
+    @mock.patch(
+        "uaclient.system.which", side_effect=["/path/to/exe", "/path/to/exe"]
+    )
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
     @mock.patch(
         M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
@@ -967,14 +661,15 @@ class TestLivepatchEntitlementEnable:
             ),
             mock.call(
                 [
-                    LIVEPATCH_CMD,
+                    livepatch.LIVEPATCH_CMD,
                     "config",
                     "remote-server=https://alt.livepatch.com",
                 ],
                 capture=True,
             ),
             mock.call(
-                [LIVEPATCH_CMD, "enable", "livepatch-token"], capture=True
+                [livepatch.LIVEPATCH_CMD, "enable", "livepatch-token"],
+                capture=True,
             ),
         ]
         assert subp_no_livepatch_disable == m_subp.call_args_list
@@ -1037,7 +732,7 @@ class TestLivepatchEntitlementEnable:
         caplog_text,
         event,
     ):
-        m_which.side_effect = [True, False]
+        m_which.side_effect = ["/path/to/exe", None]
         m_is_installed.return_value = True
 
         stderr_msg = (
@@ -1082,28 +777,71 @@ class TestLivepatchEntitlementEnable:
         assert m_snap_proxy.call_count == 1
         assert m_livepatch_proxy.call_count == 1
 
+    @mock.patch("uaclient.entitlements.livepatch.apt.run_apt_update_command")
+    @mock.patch("uaclient.system.which")
+    @mock.patch("uaclient.system.subp")
+    def test_enable_raise_exception_when_snapd_cant_be_installed(
+        self,
+        m_subp,
+        m_which,
+        _m_apt_update,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
+        _m_is_installed,
+        entitlement,
+    ):
+        m_which.side_effect = [False, True]
+
+        m_subp.side_effect = exceptions.ProcessExecutionError(
+            cmd="apt-get install --assume-yes snapd",
+            exit_code=-1,
+        )
+
+        with mock.patch.object(entitlement, "can_enable") as m_can_enable:
+            m_can_enable.return_value = (True, None)
+            with mock.patch.object(
+                entitlement, "setup_livepatch_config"
+            ) as m_setup_livepatch:
+                with pytest.raises(exceptions.CannotInstallSnapdError):
+                    entitlement.enable()
+
+            assert 1 == m_can_enable.call_count
+            assert 0 == m_setup_livepatch.call_count
+
+        assert m_validate_proxy.call_count == 0
+        assert m_snap_proxy.call_count == 0
+        assert m_livepatch_proxy.call_count == 0
+
+    @mock.patch("uaclient.entitlements.livepatch.apt.run_apt_update_command")
     @mock.patch("uaclient.system.which")
     @mock.patch("uaclient.system.subp")
     def test_enable_raise_exception_for_unexpected_error_on_snapd_wait(
         self,
         m_subp,
         m_which,
+        _m_apt_update,
         m_livepatch_proxy,
         m_snap_proxy,
         m_validate_proxy,
         m_is_installed,
         entitlement,
     ):
-        m_which.side_effect = [False, True]
+        m_which.side_effect = [None, "/path/to/exe"]
         m_is_installed.return_value = True
         stderr_msg = "test error"
 
-        m_subp.side_effect = exceptions.ProcessExecutionError(
-            cmd="snapd wait system seed.loaded",
-            exit_code=-1,
-            stdout="",
-            stderr=stderr_msg,
-        )
+        # No side effect when trying to install snapd,
+        # Exception when trying to wait
+        m_subp.side_effect = [
+            None,
+            exceptions.ProcessExecutionError(
+                cmd="snapd wait system seed.loaded",
+                exit_code=-1,
+                stdout="",
+                stderr=stderr_msg,
+            ),
+        ]
 
         with mock.patch.object(entitlement, "can_enable") as m_can_enable:
             m_can_enable.return_value = (True, None)
@@ -1126,7 +864,7 @@ class TestLivepatchEntitlementEnable:
 
 
 class TestLivepatchApplicationStatus:
-    @pytest.mark.parametrize("which_result", ((True), (False)))
+    @pytest.mark.parametrize("which_result", (("/path/to/exe"), (None)))
     @pytest.mark.parametrize("subp_raise_exception", ((True), (False)))
     @mock.patch("uaclient.system.which")
     @mock.patch("uaclient.system.subp")
@@ -1151,7 +889,7 @@ class TestLivepatchApplicationStatus:
             assert details is None
 
     @mock.patch("time.sleep")
-    @mock.patch("uaclient.system.which", return_value=True)
+    @mock.patch("uaclient.system.which", return_value="/path/to/exe")
     def test_status_command_retry_on_application_status(
         self, m_which, m_sleep, entitlement
     ):
