@@ -35,22 +35,10 @@ class RepoTestEntitlement(RepoEntitlement):
     repo_key_file = "test.gpg"
 
 
-class RepoTestEntitlementDisableAptAuthOnly(RepoTestEntitlement):
-    disable_apt_auth_only = True
-
-
-class RepoTestEntitlementDisableAptAuthOnlyFalse(RepoTestEntitlement):
-    disable_apt_auth_only = False
-
-
-class RepoTestEntitlementRepoPinInt(RepoTestEntitlement):
+class RepoTestEntitlementRepoWithPin(RepoTestEntitlement):
     repo_pin_priority = 1000
 
 
-class RepoTestEntitlementRepoPinNever(RepoTestEntitlement):
-    repo_pin_priority = "never"
-
-
 @pytest.fixture
 def entitlement(entitlement_factory):
     return entitlement_factory(
@@ -386,45 +374,6 @@ class TestProcessContractDeltas:
 
 
 class TestRepoEnable:
-    @pytest.mark.parametrize(
-        "pre_enable_msg, output, perform_enable_call_count",
-        (
-            (["msg1", (lambda: False, {}), "msg2"], "msg1\n", 0),
-            (["msg1", (lambda: True, {}), "msg2"], "msg1\nmsg2\n", 1),
-        ),
-    )
-    @mock.patch.object(
-        RepoTestEntitlement,
-        "_perform_enable",
-        return_value=False,
-    )
-    @mock.patch.object(
-        RepoTestEntitlement,
-        "can_enable",
-        return_value=(True, None),
-    )
-    def test_enable_can_exit_on_pre_enable_messaging_hooks(
-        self,
-        _m_can_enable,
-        m_perform_enable,
-        pre_enable_msg,
-        output,
-        perform_enable_call_count,
-        entitlement,
-        capsys,
-        event,
-    ):
-        with mock.patch(
-            M_PATH + "RepoEntitlement.messaging",
-            new_callable=mock.PropertyMock,
-        ) as m_messaging:
-            m_messaging.return_value = {"pre_enable": pre_enable_msg}
-            with mock.patch.object(type(entitlement), "packages", []):
-                entitlement.enable()
-        stdout, _ = capsys.readouterr()
-        assert output == stdout
-        assert perform_enable_call_count == m_perform_enable.call_count
-
     @pytest.mark.parametrize(
         "pre_disable_msg,post_disable_msg,output,retval",
         (
@@ -484,12 +433,11 @@ class TestRepoEnable:
     @pytest.mark.parametrize("should_reboot", (False, True))
     @pytest.mark.parametrize("with_pre_install_msg", (False, True))
     @pytest.mark.parametrize("packages", (["a"], [], None))
-    @mock.patch("os.getuid", return_value=0)
     @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "system.should_reboot")
     @mock.patch(M_PATH + "system.subp", return_value=("", ""))
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
-    @mock.patch(M_PATH + "os.path.exists", return_value=True)
+    @mock.patch(M_PATH + "exists", return_value=True)
     @mock.patch(M_PATH + "system.get_platform_info")
     @mock.patch.object(
         RepoTestEntitlement, "can_enable", return_value=(True, None)
@@ -503,7 +451,6 @@ class TestRepoEnable:
         m_subp,
         m_should_reboot,
         m_setup_apt_proxy,
-        _m_getuid,
         entitlement,
         capsys,
         caplog_text,
@@ -631,25 +578,57 @@ class TestRepoEnable:
 
 class TestPerformEnable:
     @pytest.mark.parametrize(
-        "supports_access_only, access_only, expected_install_calls",
-        (
-            (False, False, [mock.call()]),
-            (False, True, [mock.call()]),
-            (True, False, [mock.call()]),
-            (True, True, []),
-        ),
+        [
+            "supports_access_only",
+            "access_only",
+            "expected_setup_apt_calls",
+            "expected_install_calls",
+            "expected_check_for_reboot_calls",
+        ],
+        [
+            (
+                False,
+                False,
+                [mock.call(silent=mock.ANY)],
+                [mock.call()],
+                [mock.call(operation="install")],
+            ),
+            (
+                False,
+                True,
+                [mock.call(silent=mock.ANY)],
+                [mock.call()],
+                [mock.call(operation="install")],
+            ),
+            (
+                True,
+                False,
+                [mock.call(silent=mock.ANY)],
+                [mock.call()],
+                [mock.call(operation="install")],
+            ),
+            (
+                True,
+                True,
+                [mock.call(silent=mock.ANY)],
+                [],
+                [],
+            ),
+        ],
     )
     @mock.patch(M_PATH + "RepoEntitlement._check_for_reboot_msg")
     @mock.patch(M_PATH + "RepoEntitlement.install_packages")
     @mock.patch(M_PATH + "RepoEntitlement.setup_apt_config")
-    def test_access_only_skips_install(
+    def test_perform_enable(
         self,
-        _m_setup_apt_config,
+        m_setup_apt_config,
         m_install_packages,
-        _m_check_for_reboot_msg,
+        m_check_for_reboot_msg,
         supports_access_only,
         access_only,
+        expected_setup_apt_calls,
         expected_install_calls,
+        expected_check_for_reboot_calls,
         entitlement_factory,
     ):
         with mock.patch.object(
@@ -660,8 +639,15 @@ class TestPerformEnable:
                 affordances={"series": ["xenial"]},
                 access_only=access_only,
             )
-            entitlement._perform_enable()
+            assert entitlement._perform_enable(silent=True) is True
+            assert (
+                m_setup_apt_config.call_args_list == expected_setup_apt_calls
+            )
             assert m_install_packages.call_args_list == expected_install_calls
+            assert (
+                m_check_for_reboot_msg.call_args_list
+                == expected_check_for_reboot_calls
+            )
 
 
 class TestRemoveAptConfig:
@@ -688,7 +674,7 @@ class TestRemoveAptConfig:
     @mock.patch(M_PATH + "apt.remove_apt_list_files")
     @mock.patch(M_PATH + "apt.run_apt_command")
     @mock.patch(M_PATH + "system.get_platform_info")
-    def test_disable_apt_auth_only_false_removes_all_apt_config(
+    def test_disable_removes_all_apt_config(
         self,
         m_get_platform,
         _m_run_apt_command,
@@ -700,7 +686,7 @@ class TestRemoveAptConfig:
         m_get_platform.return_value = {"series": "xenial"}
 
         entitlement = entitlement_factory(
-            RepoTestEntitlementDisableAptAuthOnlyFalse,
+            RepoTestEntitlement,
             affordances={"series": ["xenial"]},
         )
         entitlement.remove_apt_config()
@@ -716,67 +702,7 @@ class TestRemoveAptConfig:
             )
         ] == m_remove_auth_apt_repo.call_args_list
 
-    @mock.patch(M_PATH + "apt.remove_repo_from_apt_auth_file")
-    @mock.patch(M_PATH + "apt.restore_commented_apt_list_file")
-    @mock.patch(M_PATH + "apt.run_apt_command")
-    @mock.patch(M_PATH + "system.get_platform_info")
-    def test_disable_apt_auth_only_removes_authentication_for_repo(
-        self,
-        m_get_platform,
-        _m_run_apt_command,
-        m_restore_commented_apt_list_file,
-        m_remove_repo_from_apt_auth_file,
-        entitlement_factory,
-    ):
-        """Remove APT authentication for repos with disable_apt_auth_only.
-
-        Any commented APT list entries are restored to uncommented lines.
-        """
-        m_get_platform.return_value = {"series": "xenial"}
-        entitlement = entitlement_factory(
-            RepoTestEntitlementDisableAptAuthOnly,
-            affordances={"series": ["xenial"]},
-        )
-        entitlement.remove_apt_config()
-
-        assert [
-            mock.call("http://REPOTEST")
-        ] == m_remove_repo_from_apt_auth_file.call_args_list
-        assert [
-            mock.call("/etc/apt/sources.list.d/ubuntu-repotest.list")
-        ] == m_restore_commented_apt_list_file.call_args_list
-
-    @mock.patch(M_PATH + "apt.add_ppa_pinning")
-    @mock.patch(M_PATH + "apt.remove_auth_apt_repo")
-    @mock.patch(M_PATH + "apt.remove_apt_list_files")
-    @mock.patch(M_PATH + "apt.run_apt_command")
-    @mock.patch(M_PATH + "system.get_platform_info")
-    def test_repo_pin_priority_never_configures_repo_pinning_on_remove(
-        self,
-        m_get_platform,
-        _m_run_apt_command,
-        _m_remove_apt_list_files,
-        _m_remove_auth_apt_repo,
-        m_add_ppa_pinning,
-        entitlement_factory,
-    ):
-        """Pin the repo 'never' when repo_pin_priority is never."""
-        m_get_platform.return_value = {"series": "xenial"}
-
-        entitlement = entitlement_factory(
-            RepoTestEntitlementRepoPinNever, affordances={"series": ["xenial"]}
-        )
-        entitlement.remove_apt_config()
-        assert [
-            mock.call(
-                "/etc/apt/preferences.d/ubuntu-repotest",
-                "http://REPOTEST",
-                None,
-                "never",
-            )
-        ] == m_add_ppa_pinning.call_args_list
-
-    @mock.patch(M_PATH + "system.remove_file")
+    @mock.patch(M_PATH + "system.ensure_file_absent")
     @mock.patch(M_PATH + "apt.remove_auth_apt_repo")
     @mock.patch(M_PATH + "apt.remove_apt_list_files")
     @mock.patch(M_PATH + "apt.run_apt_command")
@@ -789,21 +715,21 @@ class TestRemoveAptConfig:
         _m_run_apt_command,
         _m_remove_apt_list_files,
         _m_remove_auth_apt_repo,
-        m_remove_file,
+        m_ensure_file_absent,
         entitlement_factory,
     ):
         """Remove apt preferences file when repo_pin_priority is an int."""
         m_get_platform.return_value = {"series": "xenial"}
 
         entitlement = entitlement_factory(
-            RepoTestEntitlementRepoPinInt, affordances={"series": ["xenial"]}
+            RepoTestEntitlementRepoWithPin, affordances={"series": ["xenial"]}
         )
 
         assert 1000 == entitlement.repo_pin_priority
         entitlement.remove_apt_config()
         assert [
             mock.call("/etc/apt/preferences.d/ubuntu-repotest")
-        ] == m_remove_file.call_args_list
+        ] == m_ensure_file_absent.call_args_list
 
 
 class TestSetupAptConfig:
@@ -923,7 +849,7 @@ class TestSetupAptConfig:
         Presence is determined based on checking known files from those debs.
         It avoids a costly round-trip shelling out to call dpkg -l.
         """
-        with mock.patch(M_PATH + "os.path.exists") as m_exists:
+        with mock.patch(M_PATH + "exists") as m_exists:
             m_exists.return_value = False
             entitlement.setup_apt_config()
         assert [
@@ -943,7 +869,7 @@ class TestSetupAptConfig:
         """Raise error when repo_pin_priority is set and origin is None."""
         m_get_platform_info.return_value = {"series": "xenial"}
         entitlement = entitlement_factory(
-            RepoTestEntitlementRepoPinNever, affordances={"series": ["xenial"]}
+            RepoTestEntitlementRepoWithPin, affordances={"series": ["xenial"]}
         )
         with pytest.raises(exceptions.UserFacingError) as excinfo:
             entitlement.setup_apt_config()
@@ -952,39 +878,6 @@ class TestSetupAptConfig:
             in str(excinfo.value)
         )
 
-    @mock.patch("uaclient.apt.setup_apt_proxy")
-    @mock.patch(M_PATH + "system.remove_file")
-    @mock.patch(M_PATH + "apt.add_auth_apt_repo")
-    @mock.patch(M_PATH + "apt.run_apt_command")
-    @mock.patch(M_PATH + "system.get_platform_info")
-    @mock.patch(M_PATH + "contract.apply_contract_overrides")
-    def test_setup_with_repo_pin_priority_never_removes_apt_preferences_file(
-        self,
-        _m_contract_overrides,
-        m_get_platform_info,
-        m_run_apt_command,
-        m_add_auth_repo,
-        m_remove_file,
-        _m_setup_apt_proxy,
-        entitlement_factory,
-    ):
-        """When repo_pin_priority is never, disable repo in apt preferences."""
-        m_get_platform_info.return_value = {"series": "xenial"}
-        entitlement = entitlement_factory(
-            RepoTestEntitlementRepoPinNever, affordances={"series": ["xenial"]}
-        )
-        entitlement.origin = "RepoTestOrigin"  # don't error on origin = None
-        with mock.patch(M_PATH + "os.path.exists") as m_exists:
-            m_exists.return_value = True
-            entitlement.setup_apt_config()
-        assert [
-            mock.call("/usr/lib/apt/methods/https"),
-            mock.call("/usr/sbin/update-ca-certificates"),
-        ] == m_exists.call_args_list
-        assert [
-            mock.call("/etc/apt/preferences.d/ubuntu-repotest")
-        ] == m_remove_file.call_args_list
-
     @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "apt.run_apt_update_command")
@@ -1004,10 +897,10 @@ class TestSetupAptConfig:
         """When repo_pin_priority is an int, set pin in apt preferences."""
         m_get_platform_info.return_value = {"series": "xenial"}
         entitlement = entitlement_factory(
-            RepoTestEntitlementRepoPinInt, affordances={"series": ["xenial"]}
+            RepoTestEntitlementRepoWithPin, affordances={"series": ["xenial"]}
         )
         entitlement.origin = "RepoTestOrigin"  # don't error on origin = None
-        with mock.patch(M_PATH + "os.path.exists") as m_exists:
+        with mock.patch(M_PATH + "exists") as m_exists:
             m_exists.return_value = True  # Skip prerequisite pkg installs
             entitlement.setup_apt_config()
         assert [
@@ -1054,6 +947,7 @@ class TestCheckAptURLIsApplied:
 
 class TestApplicationStatus:
     # TODO: Write tests for all functionality
+    # 🤔
 
     def test_missing_aptURL(self, entitlement_factory):
         # Make aptURL missing
@@ -1068,26 +962,25 @@ class TestApplicationStatus:
         )
 
     @pytest.mark.parametrize(
-        "pin,policy_url,enabled",
+        "policy_url,enabled",
         (
-            (500, "https://esm.ubuntu.com/apps/ubuntu", False),
-            (-32768, "https://esm.ubuntu.com/ubuntu", False),
-            (500, "https://esm.ubuntu.com/ubuntu", True),
+            ("https://esm.ubuntu.com/apps/ubuntu", False),
+            ("https://esm.ubuntu.com/ubuntu", True),
         ),
     )
     @mock.patch(M_PATH + "apt.get_apt_cache_policy")
     def test_enabled_status_by_apt_policy(
-        self, m_run_apt_policy, pin, policy_url, enabled, entitlement_factory
+        self, m_run_apt_policy, policy_url, enabled, entitlement_factory
     ):
-        """Report ENABLED when apt-policy lists specific aptURL and 500 pin."""
+        """Report ENABLED when apt-policy lists specific aptURL."""
         entitlement = entitlement_factory(
             RepoTestEntitlement,
             directives={"aptURL": "https://esm.ubuntu.com"},
         )
 
         policy_lines = [
-            "{pin} {policy_url} bionic-security/main amd64 Packages".format(
-                pin=pin, policy_url=policy_url
+            "500 {policy_url} bionic-security/main amd64 Packages".format(
+                policy_url=policy_url
             ),
             " release v=18.04,o=UbuntuESMApps,...,n=bionic,l=UbuntuESMApps",
             "  origin esm.ubuntu.com",
