@@ -19,6 +19,7 @@ from uaclient.apt import (
     APT_PROXY_CONF_FILE,
     APT_RETRIES,
     KEYRINGS_DIR,
+    PreserveAptCfg,
     add_apt_auth_conf_entry,
     add_auth_apt_repo,
     add_ppa_pinning,
@@ -28,16 +29,18 @@ from uaclient.apt import (
     find_apt_list_files,
     get_apt_cache_policy,
     get_apt_cache_time,
-    get_installed_packages,
+    get_apt_config_values,
+    get_installed_packages_names,
     is_installed,
     remove_apt_list_files,
     remove_auth_apt_repo,
     remove_repo_from_apt_auth_file,
-    restore_commented_apt_list_file,
     run_apt_update_command,
     setup_apt_proxy,
+    update_esm_caches,
 )
 from uaclient.entitlements.base import UAEntitlement
+from uaclient.entitlements.entitlement_status import ApplicationStatus
 from uaclient.entitlements.repo import RepoEntitlement
 from uaclient.entitlements.tests.test_repo import RepoTestEntitlement
 
@@ -590,16 +593,16 @@ class TestCleanAptFiles:
         return TestRepo
 
     @mock.patch("os.path.exists", return_value=True)
-    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.system.ensure_file_absent")
     def test_removals_for_repo_entitlements(
-        self, m_remove_file, _m_path_exists
+        self, m_ensure_file_absent, _m_path_exists
     ):
         m_entitlements = mock.Mock()
         m_entitlements.ENTITLEMENT_CLASSES = [RepoTestEntitlement]
 
         clean_apt_files(_entitlements=m_entitlements)
 
-        assert 2 == m_remove_file.call_count
+        assert 2 == m_ensure_file_absent.call_count
 
     def test_files_for_all_series_removed(self, mock_apt_entitlement, tmpdir):
         m_entitlements = mock.Mock()
@@ -653,9 +656,9 @@ def remove_auth_apt_repo_kwargs(request):
 class TestRemoveAuthAptRepo:
     @mock.patch("uaclient.apt.system.subp")
     @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
-    @mock.patch("uaclient.apt.system.remove_file")
+    @mock.patch("uaclient.apt.system.ensure_file_absent")
     def test_repo_file_deleted(
-        self, m_remove_file, _mock, __mock, remove_auth_apt_repo_kwargs
+        self, m_ensure_file_absent, _mock, __mock, remove_auth_apt_repo_kwargs
     ):
         """Ensure that repo_filename is deleted, regardless of other params."""
         repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
@@ -664,10 +667,10 @@ class TestRemoveAuthAptRepo:
             repo_filename, repo_url, **remove_auth_apt_repo_kwargs
         )
 
-        assert mock.call(repo_filename) in m_remove_file.call_args_list
+        assert mock.call(repo_filename) in m_ensure_file_absent.call_args_list
 
     @mock.patch("uaclient.apt.system.subp")
-    @mock.patch("uaclient.apt.system.remove_file")
+    @mock.patch("uaclient.apt.system.ensure_file_absent")
     @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
     def test_remove_from_auth_file_called(
         self, m_remove_repo, _mock, __mock, remove_auth_apt_repo_kwargs
@@ -683,9 +686,9 @@ class TestRemoveAuthAptRepo:
 
     @mock.patch("uaclient.apt.system.subp")
     @mock.patch("uaclient.apt.remove_repo_from_apt_auth_file")
-    @mock.patch("uaclient.apt.system.remove_file")
+    @mock.patch("uaclient.apt.system.ensure_file_absent")
     def test_keyring_file_deleted_if_given(
-        self, m_remove_file, _mock, __mock, remove_auth_apt_repo_kwargs
+        self, m_ensure_file_absent, _mock, __mock, remove_auth_apt_repo_kwargs
     ):
         """We should always delete the keyring file if it is given"""
         repo_filename, repo_url = mock.sentinel.filename, mock.sentinel.url
@@ -698,18 +701,21 @@ class TestRemoveAuthAptRepo:
         if keyring_file:
             assert (
                 mock.call(os.path.join(APT_KEYS_DIR, keyring_file))
-                in m_remove_file.call_args_list
+                in m_ensure_file_absent.call_args_list
             )
         else:
-            assert mock.call(keyring_file) not in m_remove_file.call_args_list
+            assert (
+                mock.call(keyring_file)
+                not in m_ensure_file_absent.call_args_list
+            )
 
 
 class TestRemoveRepoFromAptAuthFile:
-    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.system.ensure_file_absent")
     @mock.patch("uaclient.apt.system.write_file")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_auth_file_doesnt_exist_means_we_dont_remove_or_write_it(
-        self, m_get_apt_auth_file, m_write_file, m_remove_file, tmpdir
+        self, m_get_apt_auth_file, m_write_file, m_ensure_file_absent, tmpdir
     ):
         """If the auth file doesn't exist, we shouldn't do anything to it"""
         m_get_apt_auth_file.return_value = tmpdir.join("nonexistent").strpath
@@ -717,7 +723,7 @@ class TestRemoveRepoFromAptAuthFile:
         remove_repo_from_apt_auth_file("http://url")
 
         assert 0 == m_write_file.call_count
-        assert 0 == m_remove_file.call_count
+        assert 0 == m_ensure_file_absent.call_count
 
     @pytest.mark.parametrize("trailing_slash", (True, False))
     @pytest.mark.parametrize(
@@ -733,14 +739,14 @@ class TestRemoveRepoFromAptAuthFile:
             ),
         ),
     )
-    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.system.ensure_file_absent")
     @mock.patch("uaclient.apt.system.write_file")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_file_removal(
         self,
         m_get_apt_auth_file,
         m_write_file,
-        m_remove_file,
+        m_ensure_file_absent,
         tmpdir,
         trailing_slash,
         repo_url,
@@ -756,7 +762,9 @@ class TestRemoveRepoFromAptAuthFile:
         )
 
         assert 0 == m_write_file.call_count
-        assert [mock.call(auth_file.strpath)] == m_remove_file.call_args_list
+        assert [
+            mock.call(auth_file.strpath)
+        ] == m_ensure_file_absent.call_args_list
 
     @pytest.mark.parametrize("trailing_slash", (True, False))
     @pytest.mark.parametrize(
@@ -785,12 +793,12 @@ class TestRemoveRepoFromAptAuthFile:
             ),
         ),
     )
-    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.system.ensure_file_absent")
     @mock.patch("uaclient.apt.get_apt_auth_file_from_apt_config")
     def test_file_rewrite(
         self,
         m_get_apt_auth_file,
-        m_remove_file,
+        m_ensure_file_absent,
         tmpdir,
         repo_url,
         before_content,
@@ -806,49 +814,15 @@ class TestRemoveRepoFromAptAuthFile:
             repo_url + ("" if not trailing_slash else "/")
         )
 
-        assert 0 == m_remove_file.call_count
+        assert 0 == m_ensure_file_absent.call_count
         assert 0o600 == stat.S_IMODE(os.lstat(auth_file.strpath).st_mode)
         assert after_content == auth_file.read("rb")
 
 
-class TestRestoreCommentAptListFile:
-    @pytest.mark.parametrize(
-        "before,expected",
-        (
-            ("", ""),
-            ("some other content", "some other content"),
-            ("deb uncommented", "deb uncommented"),
-            ("# deb-src comment", "# deb-src comment"),
-            ("# deb comment", "deb comment"),
-            (
-                "deb uncommented\n# deb commented",
-                "deb uncommented\ndeb commented",
-            ),
-            (
-                "# deb commented\n# deb-src commented",
-                "deb commented\n# deb-src commented",
-            ),
-            (
-                "deb uncommented\n# deb-src commented\n"
-                "# deb commented\n# deb-src commented",
-                "deb uncommented\n# deb-src commented\n"
-                "deb commented\n# deb-src commented",
-            ),
-        ),
-    )
-    def test_permutations(self, tmpdir, before, expected):
-        list_file = tmpdir.join("list_file")
-        list_file.write(before)
-
-        restore_commented_apt_list_file(list_file.strpath)
-
-        assert expected == list_file.read()
-
-
 class TestGetInstalledPackages:
     @mock.patch("uaclient.apt.system.subp", return_value=("", ""))
     def test_correct_command_called(self, m_subp):
-        get_installed_packages()
+        get_installed_packages_names()
 
         expected_call = mock.call(["apt", "list", "--installed"])
         assert [expected_call] == m_subp.call_args_list
@@ -857,7 +831,7 @@ class TestGetInstalledPackages:
         "uaclient.apt.system.subp", return_value=("Listing... Done\n", "")
     )
     def test_empty_output_means_empty_list(self, m_subp):
-        assert [] == get_installed_packages()
+        assert [] == get_installed_packages_names()
 
     @pytest.mark.parametrize(
         "apt_list_return",
@@ -866,7 +840,7 @@ class TestGetInstalledPackages:
     @mock.patch("uaclient.apt.system.subp")
     def test_lines_are_split(self, m_subp, apt_list_return):
         m_subp.return_value = apt_list_return, ""
-        assert ["a", "b"] == get_installed_packages()
+        assert ["a", "b"] == get_installed_packages_names()
 
 
 class TestRunAptCommand:
@@ -1028,10 +1002,10 @@ class TestAptProxyConfig:
         ],
     )
     @mock.patch("uaclient.system.write_file")
-    @mock.patch("uaclient.system.remove_file")
+    @mock.patch("uaclient.system.ensure_file_absent")
     def test_setup_apt_proxy_config(
         self,
-        m_util_remove_file,
+        m_ensure_file_absent,
         m_util_write_file,
         kwargs,
         expected_remove_calls,
@@ -1041,7 +1015,7 @@ class TestAptProxyConfig:
         event,
     ):
         setup_apt_proxy(**kwargs)
-        assert expected_remove_calls == m_util_remove_file.call_args_list
+        assert expected_remove_calls == m_ensure_file_absent.call_args_list
         assert expected_write_calls == m_util_write_file.call_args_list
         out, err = capsys.readouterr()
         assert expected_out == out.strip()
@@ -1056,7 +1030,7 @@ class TestAptIsInstalled:
             (False, ("foo", "bar")),
         ),
     )
-    @mock.patch("uaclient.apt.get_installed_packages")
+    @mock.patch("uaclient.apt.get_installed_packages_names")
     def test_is_installed_pkgs(
         self, m_get_installed_pkgs, expected, installed_pkgs
     ):
@@ -1087,7 +1061,7 @@ class TestCompareVersion:
             assert expected_result is compare_versions(ver1, ver2, relation)
 
 
-class TestAptCacheTime:
+class TestAptCache:
     @pytest.mark.parametrize(
         "file_exists,expected", ((True, 1.23), (False, None))
     )
@@ -1097,3 +1071,175 @@ class TestAptCacheTime:
         m_stat.return_value.st_mtime = 1.23
         m_exists.return_value = file_exists
         assert expected == get_apt_cache_time()
+
+    @pytest.mark.parametrize(
+        "is_lts,cache_call_list",
+        ((True, [mock.call()]), (False, [])),
+    )
+    @pytest.mark.parametrize(
+        "apps_status", (ApplicationStatus.ENABLED, ApplicationStatus.DISABLED)
+    )
+    @pytest.mark.parametrize(
+        "infra_status", (ApplicationStatus.ENABLED, ApplicationStatus.DISABLED)
+    )
+    @pytest.mark.parametrize("is_esm", (True, False))
+    @pytest.mark.parametrize("can_enable_infra", ("yes", "no"))
+    @pytest.mark.parametrize("can_enable_apps", ("yes", "no"))
+    @mock.patch("uaclient.entitlements.esm.ESMAppsEntitlement")
+    @mock.patch("uaclient.entitlements.esm.ESMInfraEntitlement")
+    @mock.patch("uaclient.apt.system.is_current_series_lts")
+    @mock.patch("uaclient.apt.system.is_current_series_active_esm")
+    @mock.patch("uaclient.apt.get_esm_cache")
+    @mock.patch("uaclient.actions.status")
+    @mock.patch("apt_pkg.config")
+    @mock.patch("apt_pkg.init_config")
+    def test_update_esm_caches_based_on_lts(
+        self,
+        _m_apt_pkg_init_config,
+        _m_apt_pkg_config,
+        m_status,
+        m_esm_cache,
+        m_is_esm,
+        m_is_lts,
+        m_infra_entitlement,
+        m_apps_entitlement,
+        is_lts,
+        cache_call_list,
+        apps_status,
+        infra_status,
+        is_esm,
+        can_enable_infra,
+        can_enable_apps,
+        FakeConfig,
+    ):
+        m_status.return_value = (
+            {
+                "services": [
+                    {"name": "esm-apps", "available": can_enable_apps},
+                    {"name": "esm-infra", "available": can_enable_infra},
+                ]
+            },
+            0,
+        )
+
+        m_is_esm.return_value = is_esm
+        m_is_lts.return_value = is_lts
+
+        m_infra = mock.MagicMock()
+        m_apps = mock.MagicMock()
+
+        m_infra.application_status.return_value = (
+            infra_status,
+            "",
+        )
+
+        m_apps.application_status.return_value = (
+            apps_status,
+            "",
+        )
+
+        m_infra_entitlement.return_value = m_infra
+        m_apps_entitlement.return_value = m_apps
+
+        infra_setup_repo_count = 0
+        apps_setup_repo_count = 0
+        infra_disable_repo_count = 0
+        apps_disable_repo_count = 0
+        status_count = 0
+        status_cache_args_list = []
+
+        if is_lts:
+            status_count = 1
+            status_cache_args_list = [mock.call("status-cache")]
+            if (
+                apps_status == ApplicationStatus.DISABLED
+                and can_enable_apps == "yes"
+            ):
+                apps_setup_repo_count = 1
+            else:
+                apps_disable_repo_count = 1
+
+            if (
+                infra_status == ApplicationStatus.DISABLED
+                and is_esm
+                and can_enable_infra == "yes"
+            ):
+                infra_setup_repo_count = 1
+            elif is_esm:
+                infra_disable_repo_count = 1
+
+        cfg = FakeConfig()
+        with mock.patch.object(cfg, "read_cache", return_value=None):
+            update_esm_caches(cfg)
+            assert cfg.read_cache.call_args_list == status_cache_args_list
+
+        assert m_esm_cache.call_args_list == cache_call_list
+        assert (
+            m_infra.setup_local_esm_repo.call_count == infra_setup_repo_count
+        )
+        assert m_apps.setup_local_esm_repo.call_count == apps_setup_repo_count
+        assert (
+            m_infra.disable_local_esm_repo.call_count
+            == infra_disable_repo_count
+        )
+        assert (
+            m_apps.disable_local_esm_repo.call_count == apps_disable_repo_count
+        )
+        assert m_status.call_count == status_count
+
+
+class TestGetAptConfigValues:
+    @mock.patch("uaclient.apt._get_apt_config")
+    def test_apt_config_values(
+        self,
+        m_get_apt_config,
+    ):
+        m_dict = mock.MagicMock()
+        m_get_apt_config.return_value = m_dict
+
+        m_dict.get.side_effect = ["", "foo", "bar", ""]
+        m_dict.value_list.side_effect = [
+            "",
+            ["test1", "test2"],
+        ]
+
+        expected_return = {
+            "val1": None,
+            "val2": "foo",
+            "val3": "bar",
+            "val4": ["test1", "test2"],
+        }
+
+        assert expected_return == get_apt_config_values(
+            ["val1", "val2", "val3", "val4"]
+        )
+
+
+class TestPreserveAptCfg:
+    def test_apt_config_is_preserved(
+        self,
+        apt_pkg,
+    ):
+        class AptDict(dict):
+            def set(self, key, value):
+                super().__setitem__(key, value)
+
+        apt_cfg = AptDict()
+        apt_cfg["test"] = 1
+        apt_cfg["test1"] = [1, 2, 3]
+        apt_cfg["test2"] = {"foo": "bar"}
+
+        type(apt_pkg).config = mock.PropertyMock(return_value=apt_cfg)
+
+        def apt_func():
+            apt_cfg["test"] = 3
+            apt_cfg["test1"] = [3, 2, 1]
+            apt_cfg["test2"] = {"foo": "test"}
+            return apt_cfg
+
+        with PreserveAptCfg(apt_func):
+            pass
+
+        assert 1 == apt_cfg["test"]
+        assert [1, 2, 3] == apt_cfg["test1"]
+        assert {"foo": "bar"} == apt_cfg["test2"]
