@@ -2,27 +2,56 @@ import contextlib
 import copy
 import io
 import json
+import textwrap
 
 import mock
 import pytest
-import yaml
 
-from uaclient import event_logger, messages, status
+from uaclient import event_logger, messages, status, util
 from uaclient.cli import (
     UA_AUTH_TOKEN_URL,
     action_attach,
     attach_parser,
     get_parser,
+    main,
     main_error_handler,
 )
 from uaclient.exceptions import (
     AlreadyAttachedError,
     LockHeldError,
+    MagicAttachInvalidParam,
+    MagicAttachTokenError,
     NonRootUserError,
     UrlError,
     UserFacingError,
 )
 from uaclient.testing.fakes import FakeFile
+from uaclient.yaml import safe_dump
+
+HELP_OUTPUT = textwrap.dedent(
+    """\
+usage: pro attach <token> [flags]
+
+Attach this machine to Ubuntu Pro with a token obtained from:
+https://ubuntu.com/pro
+
+When running this command without a token, it will generate a short code
+and prompt you to attach the machine to your Ubuntu Pro account using
+a web browser.
+
+positional arguments:
+  token                 token obtained for Ubuntu Pro authentication:
+                        https://auth.contracts.canonical.com
+
+Flags:
+  -h, --help            show this help message and exit
+  --no-auto-enable      do not enable any recommended services automatically
+  --attach-config ATTACH_CONFIG
+                        use the provided attach config file instead of passing
+                        the token on the cli
+  --format {cli,json}   output enable in the specified format (default: cli)
+"""
+)
 
 M_PATH = "uaclient.cli."
 
@@ -35,8 +64,8 @@ BASIC_MACHINE_TOKEN = {
         "contractInfo": {
             "name": "mycontract",
             "id": "contract-1",
-            "createdAt": "2020-05-08T19:02:26Z",
-            "effectiveTo": "9999-12-31T00:00:00Z",
+            "createdAt": util.parse_rfc3339_date("2020-05-08T19:02:26Z"),
+            "effectiveTo": util.parse_rfc3339_date("9999-12-31T00:00:00Z"),
             "resourceEntitlements": [],
             "products": ["free"],
         },
@@ -78,10 +107,11 @@ ENTITLED_MACHINE_TOKEN["machineTokenInfo"]["contractInfo"][
 ] = [ENTITLED_EXAMPLE_ESM_RESOURCE]
 
 
-@mock.patch(M_PATH + "os.getuid")
-def test_non_root_users_are_rejected(getuid, FakeConfig, capsys, event):
+@mock.patch(M_PATH + "util.we_are_currently_root", return_value=False)
+def test_non_root_users_are_rejected(
+    m_we_are_currently_root, FakeConfig, capsys, event
+):
     """Check that a UID != 0 will receive a message and exit non-zero"""
-    getuid.return_value = 1
 
     cfg = FakeConfig()
     with pytest.raises(NonRootUserError):
@@ -112,10 +142,8 @@ def test_non_root_users_are_rejected(getuid, FakeConfig, capsys, event):
     assert expected == json.loads(capsys.readouterr()[0])
 
 
-# For all of these tests we want to appear as root, so mock on the class
-@mock.patch(M_PATH + "os.getuid", return_value=0)
 class TestActionAttach:
-    def test_already_attached(self, _m_getuid, capsys, FakeConfig, event):
+    def test_already_attached(self, capsys, FakeConfig, event):
         """Check that an already-attached machine emits message and exits 0"""
         account_name = "test_account"
         cfg = FakeConfig.for_attached_machine(
@@ -152,7 +180,11 @@ class TestActionAttach:
 
     @mock.patch("uaclient.system.subp")
     def test_lock_file_exists(
-        self, m_subp, _m_getuid, capsys, FakeConfig, event
+        self,
+        m_subp,
+        capsys,
+        FakeConfig,
+        event,
     ):
         """Check when an operation holds a lock file, attach cannot run."""
         cfg = FakeConfig()
@@ -196,48 +228,6 @@ class TestActionAttach:
         }
         assert expected == json.loads(capsys.readouterr()[0])
 
-    def test_token_is_a_required_argument(
-        self, _m_getuid, FakeConfig, capsys, event
-    ):
-        """When missing the required token argument, raise a UserFacingError"""
-        args = mock.MagicMock(token=None, attach_config=None)
-        cfg = FakeConfig()
-        with pytest.raises(UserFacingError) as e:
-            action_attach(args, cfg=cfg)
-        assert messages.ATTACH_REQUIRES_TOKEN.msg == str(e.value.msg)
-
-        args = mock.MagicMock()
-        args.token = None
-        args.attach_config = None
-        with pytest.raises(SystemExit):
-            with mock.patch.object(
-                event, "_event_logger_mode", event_logger.EventLoggerMode.JSON
-            ):
-                with mock.patch.object(
-                    cfg, "check_lock_info"
-                ) as m_check_lock_info:
-                    m_check_lock_info.return_value = (0, "lock_holder")
-                    main_error_handler(action_attach)(args, cfg)
-
-        expected_msg = messages.ATTACH_REQUIRES_TOKEN
-        expected = {
-            "_schema_version": event_logger.JSON_SCHEMA_VERSION,
-            "result": "failure",
-            "errors": [
-                {
-                    "message": expected_msg.msg,
-                    "message_code": expected_msg.name,
-                    "service": None,
-                    "type": "system",
-                }
-            ],
-            "failed_services": [],
-            "needs_reboot": False,
-            "processed_services": [],
-            "warnings": [],
-        }
-        assert expected == json.loads(capsys.readouterr()[0])
-
     @pytest.mark.parametrize(
         "error_class, error_str",
         (
@@ -246,18 +236,17 @@ class TestActionAttach:
         ),
     )
     @mock.patch("uaclient.system.should_reboot", return_value=False)
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     @mock.patch("uaclient.status.get_available_resources")
-    @mock.patch("uaclient.jobs.update_messaging.update_apt_and_motd_messages")
+    @mock.patch("uaclient.jobs.update_messaging.update_motd_messages")
     @mock.patch(M_PATH + "contract.request_updated_contract")
     def test_status_updated_when_auto_enable_fails(
         self,
         request_updated_contract,
         m_update_apt_and_motd_msgs,
         _m_get_available_resources,
-        _m_should_reboot,
         _m_remove_notice,
-        _m_get_uid,
+        _m_should_reboot,
         error_class,
         error_str,
         FakeConfig,
@@ -288,8 +277,8 @@ class TestActionAttach:
         assert [mock.call(cfg)] == m_update_apt_and_motd_msgs.call_args_list
 
     @mock.patch("uaclient.system.should_reboot", return_value=False)
-    @mock.patch("uaclient.files.NoticeFile.remove")
-    @mock.patch("uaclient.jobs.update_messaging.update_apt_and_motd_messages")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
+    @mock.patch("uaclient.jobs.update_messaging.update_motd_messages")
     @mock.patch(
         M_PATH + "contract.UAContractClient.request_contract_machine_attach"
     )
@@ -301,9 +290,8 @@ class TestActionAttach:
         m_status,
         contract_machine_attach,
         m_update_apt_and_motd_msgs,
-        _m_should_reboot,
         _m_remove_notice,
-        _m_getuid,
+        _m_should_reboot,
         FakeConfig,
         event,
     ):
@@ -363,16 +351,15 @@ class TestActionAttach:
 
     @pytest.mark.parametrize("auto_enable", (True, False))
     @mock.patch("uaclient.system.should_reboot", return_value=False)
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     @mock.patch("uaclient.status.get_available_resources")
-    @mock.patch("uaclient.jobs.update_messaging.update_apt_and_motd_messages")
+    @mock.patch("uaclient.jobs.update_messaging.update_motd_messages")
     def test_auto_enable_passed_through_to_request_updated_contract(
         self,
         m_update_apt_and_motd_msgs,
         _m_get_available_resources,
-        _m_should_reboot,
         _m_remove_notice,
-        _m_get_uid,
+        _m_should_reboot,
         auto_enable,
         FakeConfig,
     ):
@@ -392,7 +379,8 @@ class TestActionAttach:
         assert [mock.call(cfg)] == m_update_apt_and_motd_msgs.call_args_list
 
     def test_attach_config_and_token_mutually_exclusive(
-        self, _m_getuid, FakeConfig
+        self,
+        FakeConfig,
     ):
         args = mock.MagicMock(
             token="something", attach_config=FakeFile("something")
@@ -405,11 +393,14 @@ class TestActionAttach:
     @mock.patch(M_PATH + "_post_cli_attach")
     @mock.patch(M_PATH + "actions.attach_with_token")
     def test_token_from_attach_config(
-        self, m_attach_with_token, _m_post_cli_attach, _m_getuid, FakeConfig
+        self,
+        m_attach_with_token,
+        _m_post_cli_attach,
+        FakeConfig,
     ):
         args = mock.MagicMock(
             token=None,
-            attach_config=FakeFile(yaml.dump({"token": "faketoken"})),
+            attach_config=FakeFile(safe_dump({"token": "faketoken"})),
         )
         cfg = FakeConfig()
         action_attach(args, cfg=cfg)
@@ -418,12 +409,15 @@ class TestActionAttach:
         ] == m_attach_with_token.call_args_list
 
     def test_attach_config_invalid_config(
-        self, _m_getuid, FakeConfig, capsys, event
+        self,
+        FakeConfig,
+        capsys,
+        event,
     ):
         args = mock.MagicMock(
             token=None,
             attach_config=FakeFile(
-                yaml.dump({"token": "something", "enable_services": "cis"}),
+                safe_dump({"token": "something", "enable_services": "cis"}),
                 name="fakename",
             ),
         )
@@ -433,7 +427,7 @@ class TestActionAttach:
         assert "Error while reading fakename: " in e.value.msg
 
         args.attach_config = FakeFile(
-            yaml.dump({"token": "something", "enable_services": "cis"}),
+            safe_dump({"token": "something", "enable_services": "cis"}),
             name="fakename",
         )
         with pytest.raises(SystemExit):
@@ -478,16 +472,17 @@ class TestActionAttach:
     @mock.patch("uaclient.util.handle_unicode_characters")
     @mock.patch("uaclient.status.format_tabular")
     @mock.patch(M_PATH + "actions.status")
+    @mock.patch("uaclient.daemon.cleanup")
     @mock.patch("uaclient.daemon.stop")
     def test_attach_config_enable_services(
         self,
         _m_daemon_stop,
+        _m_daemon_cleanup,
         m_status,
         m_format_tabular,
         m_handle_unicode,
         m_attach_with_token,
         m_enable,
-        _m_getuid,
         auto_enable,
         FakeConfig,
         event,
@@ -500,7 +495,7 @@ class TestActionAttach:
         args = mock.MagicMock(
             token=None,
             attach_config=FakeFile(
-                yaml.dump({"token": "faketoken", "enable_services": ["cis"]})
+                safe_dump({"token": "faketoken", "enable_services": ["cis"]})
             ),
             auto_enable=auto_enable,
         )
@@ -516,7 +511,7 @@ class TestActionAttach:
             assert [] == m_enable.call_args_list
 
         args.attach_config = FakeFile(
-            yaml.dump({"token": "faketoken", "enable_services": ["cis"]})
+            safe_dump({"token": "faketoken", "enable_services": ["cis"]})
         )
 
         fake_stdout = io.StringIO()
@@ -541,7 +536,7 @@ class TestActionAttach:
     @mock.patch("uaclient.contract.process_entitlement_delta")
     @mock.patch("uaclient.contract.apply_contract_overrides")
     @mock.patch("uaclient.contract.UAContractClient.request_url")
-    @mock.patch("uaclient.jobs.update_messaging.update_apt_and_motd_messages")
+    @mock.patch("uaclient.jobs.update_messaging.update_motd_messages")
     def test_attach_when_one_service_fails_to_enable(
         self,
         _m_update_messages,
@@ -549,7 +544,6 @@ class TestActionAttach:
         _m_apply_contract_overrides,
         m_process_entitlement_delta,
         m_enable_order,
-        _m_getuid,
         FakeConfig,
         event,
     ):
@@ -569,7 +563,9 @@ class TestActionAttach:
                     "accountInfo": {
                         "id": "acct-1",
                         "name": "acc-name",
-                        "createdAt": "2019-06-14T06:45:50Z",
+                        "createdAt": util.parse_rfc3339_date(
+                            "2019-06-14T06:45:50Z"
+                        ),
                         "externalAccountIDs": [
                             {"IDs": ["id1"], "origin": "AWS"}
                         ],
@@ -620,9 +616,53 @@ class TestActionAttach:
         }
         assert expected == json.loads(fake_stdout.getvalue())
 
+    @mock.patch(M_PATH + "_initiate")
+    @mock.patch(M_PATH + "_wait")
+    @mock.patch(M_PATH + "_revoke")
+    def test_magic_attach_revoke_token_if_wait_fails(
+        self,
+        m_revoke,
+        m_wait,
+        m_initiate,
+        FakeConfig,
+    ):
+        m_initiate.return_value = mock.MagicMock(
+            token="token", user_code="user_code"
+        )
+        m_wait.side_effect = MagicAttachTokenError()
+        m_args = mock.MagicMock(token=None, attach_config=None)
+
+        with pytest.raises(MagicAttachTokenError):
+            action_attach(args=m_args, cfg=FakeConfig())
+
+        assert 1 == m_initiate.call_count
+        assert 1 == m_wait.call_count
+        assert 1 == m_revoke.call_count
+
+    def test_magic_attach_fails_if_format_json_param_used(self, FakeConfig):
+        m_args = mock.MagicMock(token=None, attach_config=None, format="json")
+
+        with pytest.raises(MagicAttachInvalidParam) as exc_info:
+            action_attach(args=m_args, cfg=FakeConfig())
+
+        assert (
+            "This attach flow does not support --format with value: json"
+        ) == exc_info.value.msg
+
 
 @mock.patch(M_PATH + "contract.get_available_resources")
 class TestParser:
+    def test_attach_help(self, _m_resources, capsys, FakeConfig):
+        with pytest.raises(SystemExit):
+            with mock.patch("sys.argv", ["/usr/bin/pro", "attach", "--help"]):
+                with mock.patch(
+                    "uaclient.config.UAConfig",
+                    return_value=FakeConfig(),
+                ):
+                    main()
+        out, _err = capsys.readouterr()
+        assert HELP_OUTPUT in out
+
     def test_attach_parser_usage(self, _m_resources):
         parser = attach_parser(mock.Mock())
         assert "pro attach <token> [flags]" == parser.usage
