@@ -3,6 +3,7 @@ import pytest
 
 from uaclient import exceptions, messages
 from uaclient.cli import action_refresh, main
+from uaclient.files.notices import Notice
 
 HELP_OUTPUT = """\
 usage: pro refresh [contract|config|messages] [flags]
@@ -26,10 +27,9 @@ Flags:
 """
 
 
-@mock.patch("os.getuid", return_value=0)
 class TestActionRefresh:
     @mock.patch("uaclient.cli.contract.get_available_resources")
-    def test_refresh_help(self, _m_resources, _getuid, capsys, FakeConfig):
+    def test_refresh_help(self, _m_resources, capsys, FakeConfig):
         with pytest.raises(SystemExit):
             with mock.patch("sys.argv", ["/usr/bin/ua", "refresh", "--help"]):
                 with mock.patch(
@@ -40,9 +40,11 @@ class TestActionRefresh:
         out, _err = capsys.readouterr()
         assert HELP_OUTPUT in out
 
-    def test_non_root_users_are_rejected(self, getuid, FakeConfig):
+    @mock.patch("uaclient.util.we_are_currently_root", return_value=False)
+    def test_non_root_users_are_rejected(
+        self, we_are_currently_root, FakeConfig
+    ):
         """Check that a UID != 0 will receive a message and exit non-zero"""
-        getuid.return_value = 1
 
         cfg = FakeConfig.for_attached_machine()
         with pytest.raises(exceptions.NonRootUserError):
@@ -52,16 +54,17 @@ class TestActionRefresh:
         "target, expect_unattached_error",
         [(None, True), ("contract", True), ("config", False)],
     )
-    @mock.patch("uaclient.config.UAConfig.write_cfg")
     def test_not_attached_errors(
-        self, _m_write_cfg, getuid, target, expect_unattached_error, FakeConfig
+        self,
+        target,
+        expect_unattached_error,
+        FakeConfig,
     ):
         """Check that an unattached machine emits message and exits 1"""
         cfg = FakeConfig()
 
-        cfg.update_messaging_timer = 0
-        cfg.update_status_timer = 0
-        cfg.metering_timer = 0
+        cfg.user_config.update_messaging_timer = 0
+        cfg.user_config.metering_timer = 0
 
         if expect_unattached_error:
             with pytest.raises(exceptions.UnattachedError):
@@ -70,7 +73,7 @@ class TestActionRefresh:
             action_refresh(mock.MagicMock(target=target), cfg=cfg)
 
     @mock.patch("uaclient.system.subp")
-    def test_lock_file_exists(self, m_subp, _getuid, FakeConfig):
+    def test_lock_file_exists(self, m_subp, FakeConfig):
         """Check inability to refresh if operation holds lock file."""
         cfg = FakeConfig().for_attached_machine()
         cfg.write_cache("lock", "123:pro disable")
@@ -84,13 +87,12 @@ class TestActionRefresh:
 
     @mock.patch("logging.exception")
     @mock.patch("uaclient.contract.request_updated_contract")
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     def test_refresh_contract_error_on_failure_to_update_contract(
         self,
         m_remove_notice,
         request_updated_contract,
         logging_error,
-        getuid,
         FakeConfig,
     ):
         """On failure in request_updates_contract emit an error."""
@@ -109,12 +111,11 @@ class TestActionRefresh:
         ] != m_remove_notice.call_args_list
 
     @mock.patch("uaclient.contract.request_updated_contract")
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     def test_refresh_contract_happy_path(
         self,
         m_remove_notice,
         request_updated_contract,
-        getuid,
         capsys,
         FakeConfig,
     ):
@@ -128,13 +129,13 @@ class TestActionRefresh:
         assert messages.REFRESH_CONTRACT_SUCCESS in capsys.readouterr()[0]
         assert [mock.call(cfg)] == request_updated_contract.call_args_list
         assert [
-            mock.call("", messages.NOTICE_REFRESH_CONTRACT_WARNING),
-            mock.call("", "Operation in progress.*"),
+            mock.call(Notice.CONTRACT_REFRESH_WARNING),
+            mock.call(Notice.OPERATION_IN_PROGRESS),
         ] == m_remove_notice.call_args_list
 
-    @mock.patch("uaclient.cli.update_apt_and_motd_messages")
-    def test_refresh_messages_error(self, m_update_motd, getuid, FakeConfig):
-        """On failure in update_apt_and_motd_messages emit an error."""
+    @mock.patch("uaclient.cli.update_motd_messages")
+    def test_refresh_messages_error(self, m_update_motd, FakeConfig):
+        """On failure in update_motd_messages emit an error."""
         m_update_motd.side_effect = Exception("test")
 
         with pytest.raises(exceptions.UserFacingError) as excinfo:
@@ -142,17 +143,18 @@ class TestActionRefresh:
 
         assert messages.REFRESH_MESSAGES_FAILURE == excinfo.value.msg
 
+    @mock.patch("uaclient.apt_news.update_apt_news")
     @mock.patch("uaclient.jobs.update_messaging.exists", return_value=True)
     @mock.patch("logging.exception")
     @mock.patch("uaclient.system.subp")
-    @mock.patch("uaclient.cli.update_apt_and_motd_messages")
+    @mock.patch("uaclient.cli.update_motd_messages")
     def test_refresh_messages_doesnt_fail_if_update_notifier_does(
         self,
         m_update_motd,
         m_subp,
         logging_error,
         _m_path,
-        getuid,
+        _m_update_apt_news,
         capsys,
         FakeConfig,
     ):
@@ -168,27 +170,16 @@ class TestActionRefresh:
         assert [mock.call(subp_exc)] == logging_error.call_args_list
         assert messages.REFRESH_MESSAGES_SUCCESS in capsys.readouterr()[0]
 
-    @mock.patch("uaclient.jobs.update_messaging.exists", return_value=True)
-    @mock.patch("logging.exception")
-    @mock.patch("uaclient.system.subp")
-    @mock.patch("uaclient.cli.update_apt_and_motd_messages")
-    def test_refresh_messages_systemctl_error(
-        self, m_update_motd, m_subp, logging_error, _m_path, getuid, FakeConfig
-    ):
-        subp_exc = Exception("test")
-        m_subp.side_effect = ["", subp_exc]
-
-        with pytest.raises(exceptions.UserFacingError) as excinfo:
-            action_refresh(mock.MagicMock(target="messages"), cfg=FakeConfig())
-
-        assert 1 == logging_error.call_count
-        assert [mock.call(subp_exc)] == logging_error.call_args_list
-        assert messages.REFRESH_MESSAGES_FAILURE == excinfo.value.msg
-
+    @mock.patch("uaclient.apt_news.update_apt_news")
     @mock.patch("uaclient.cli.refresh_motd")
-    @mock.patch("uaclient.cli.update_apt_and_motd_messages")
+    @mock.patch("uaclient.cli.update_motd_messages")
     def test_refresh_messages_happy_path(
-        self, m_update_motd, m_refresh_motd, getuid, capsys, FakeConfig
+        self,
+        m_update_motd,
+        m_refresh_motd,
+        m_update_apt_news,
+        capsys,
+        FakeConfig,
     ):
         """On success from request_updates_contract root user can refresh."""
         cfg = FakeConfig()
@@ -198,13 +189,19 @@ class TestActionRefresh:
         assert messages.REFRESH_MESSAGES_SUCCESS in capsys.readouterr()[0]
         assert [mock.call(cfg)] == m_update_motd.call_args_list
         assert [mock.call()] == m_refresh_motd.call_args_list
+        assert 1 == m_update_motd.call_count
+        assert 1 == m_refresh_motd.call_count
+        assert 1 == m_update_apt_news.call_count
 
     @mock.patch("logging.exception")
     @mock.patch(
         "uaclient.config.UAConfig.process_config", side_effect=RuntimeError()
     )
     def test_refresh_config_error_on_failure_to_process_config(
-        self, _m_process_config, _m_logging_error, getuid, FakeConfig
+        self,
+        _m_process_config,
+        _m_logging_error,
+        FakeConfig,
     ):
         """On failure in process_config emit an error."""
 
@@ -217,7 +214,10 @@ class TestActionRefresh:
 
     @mock.patch("uaclient.config.UAConfig.process_config")
     def test_refresh_config_happy_path(
-        self, m_process_config, getuid, capsys, FakeConfig
+        self,
+        m_process_config,
+        capsys,
+        FakeConfig,
     ):
         """On success from process_config root user gets success message."""
 
@@ -228,15 +228,20 @@ class TestActionRefresh:
         assert messages.REFRESH_CONFIG_SUCCESS in capsys.readouterr()[0]
         assert [mock.call()] == m_process_config.call_args_list
 
+    @mock.patch("uaclient.apt_news.update_apt_news")
+    @mock.patch("uaclient.cli.refresh_motd")
+    @mock.patch("uaclient.cli.update_motd_messages")
     @mock.patch("uaclient.contract.request_updated_contract")
     @mock.patch("uaclient.config.UAConfig.process_config")
-    @mock.patch("uaclient.files.NoticeFile.remove")
+    @mock.patch("uaclient.files.notices.NoticesManager.remove")
     def test_refresh_all_happy_path(
         self,
         m_remove_notice,
         m_process_config,
         m_request_updated_contract,
-        getuid,
+        m_update_motd,
+        m_refresh_motd,
+        m_update_apt_news,
         capsys,
         FakeConfig,
     ):
@@ -249,9 +254,13 @@ class TestActionRefresh:
         assert 0 == ret
         assert messages.REFRESH_CONFIG_SUCCESS in out
         assert messages.REFRESH_CONTRACT_SUCCESS in out
+        assert messages.REFRESH_MESSAGES_SUCCESS in out
         assert [mock.call()] == m_process_config.call_args_list
         assert [mock.call(cfg)] == m_request_updated_contract.call_args_list
         assert [
-            mock.call("", messages.NOTICE_REFRESH_CONTRACT_WARNING),
-            mock.call("", "Operation in progress.*"),
+            mock.call(Notice.CONTRACT_REFRESH_WARNING),
+            mock.call(Notice.OPERATION_IN_PROGRESS),
         ] == m_remove_notice.call_args_list
+        assert 1 == m_update_motd.call_count
+        assert 1 == m_refresh_motd.call_count
+        assert 1 == m_update_apt_news.call_count
