@@ -17,7 +17,7 @@ M_PATH = "uaclient.cli."
 HELP_OUTPUT = textwrap.dedent(
     """\
 usage: security-status \[-h\] \[--format {json,yaml,text}\]
-                       \[--thirdparty \| --unavailable \| --esm-infra\]
+                       \[--thirdparty \| --unavailable \| --esm-infra\ \| --esm-apps]
 
 Show security updates for packages in the system, including all
 available Expanded Security Maintenance \(ESM\) related content.
@@ -45,6 +45,7 @@ complete status on Ubuntu Pro services, run 'pro status'.
   --unavailable         List and present information about unavailable
                         packages
   --esm-infra           List and present information about esm-infra packages
+  --esm-apps            List and present information about esm-apps packages
 """  # noqa
 )
 
@@ -74,7 +75,7 @@ class TestActionSecurityStatus:
         assert re.match(HELP_OUTPUT, out)
 
     @pytest.mark.parametrize("output_format", ("json", "yaml", "text"))
-    @mock.patch(M_PATH + "yaml.safe_dump")
+    @mock.patch(M_PATH + "safe_dump")
     @mock.patch(M_PATH + "json.dumps")
     def test_action_security_status(
         self,
