@@ -9,11 +9,13 @@ import textwrap
 
 import mock
 import pytest
-import yaml
 
-from uaclient import exceptions, messages, status
+from uaclient import exceptions, messages, status, util
 from uaclient.cli import action_status, get_parser, main, status_parser
+from uaclient.conftest import FakeNotice
 from uaclient.event_logger import EventLoggerMode
+from uaclient.files.notices import Notice, NoticesManager
+from uaclient.yaml import safe_load
 
 M_PATH = "uaclient.cli."
 
@@ -31,15 +33,15 @@ RESPONSE_AVAILABLE_SERVICES = [
 
 RESPONSE_CONTRACT_INFO = {
     "accountInfo": {
-        "createdAt": "2019-06-14T06:45:50Z",
+        "createdAt": util.parse_rfc3339_date("2019-06-14T06:45:50Z"),
         "id": "some_id",
         "name": "Name",
         "type": "paid",
     },
     "contractInfo": {
-        "createdAt": "2021-05-21T20:00:53Z",
+        "createdAt": util.parse_rfc3339_date("2021-05-21T20:00:53Z"),
         "createdBy": "someone",
-        "effectiveTo": "9999-12-31T00:00:00Z",
+        "effectiveTo": util.parse_rfc3339_date("9999-12-31T00:00:00Z"),
         "id": "some_id",
         "name": "Name",
         "products": ["uai-essential-virtual"],
@@ -75,13 +77,14 @@ esm-infra        yes        yes        yes           Expanded Security Maintenan
 fips             no         no         no            NIST-certified core packages
 fips-updates     no         no         no            NIST-certified core packages with priority security updates
 livepatch        yes        yes        no            Canonical Livepatch service
-realtime-kernel  no         no         no            Beta-version Ubuntu Kernel with PREEMPT_RT patches
+realtime-kernel  no         no         no            Ubuntu kernel with PREEMPT_RT patches integrated
 ros              no         no         no            Security Updates for the Robot Operating System
 ros-updates      no         no         no            All Updates for the Robot Operating System
 """  # noqa: E501
 
 SIMULATED_STATUS = """\
 SERVICE          AVAILABLE  ENTITLED   AUTO_ENABLED  DESCRIPTION
+esm-apps         yes        no         yes           Expanded Security Maintenance for Applications
 esm-infra        yes        yes        yes           Expanded Security Maintenance for Infrastructure
 livepatch        yes        yes        no            Canonical Livepatch service
 """  # noqa: E501
@@ -93,7 +96,7 @@ esm-infra        yes        Expanded Security Maintenance for Infrastructure
 fips             no         NIST-certified core packages
 fips-updates     no         NIST-certified core packages with priority security updates
 livepatch        yes        Canonical Livepatch service
-realtime-kernel  no         Beta-version Ubuntu Kernel with PREEMPT_RT patches
+realtime-kernel  no         Ubuntu kernel with PREEMPT_RT patches integrated
 ros              no         Security Updates for the Robot Operating System
 ros-updates      no         All Updates for the Robot Operating System
 
@@ -103,6 +106,7 @@ See https://ubuntu.com/pro
 
 UNATTACHED_STATUS = """\
 SERVICE          AVAILABLE  DESCRIPTION
+esm-apps         yes        Expanded Security Maintenance for Applications
 esm-infra        yes        Expanded Security Maintenance for Infrastructure
 livepatch        yes        Canonical Livepatch service
 
@@ -117,7 +121,7 @@ esm-infra        no        {dash}         Expanded Security Maintenance for Infr
 fips             no        {dash}         NIST-certified core packages
 fips-updates     no        {dash}         NIST-certified core packages with priority security updates
 livepatch        no        {dash}         Canonical Livepatch service
-realtime-kernel  no        {dash}         Beta-version Ubuntu Kernel with PREEMPT_RT patches
+realtime-kernel  no        {dash}         Ubuntu kernel with PREEMPT_RT patches integrated
 ros              no        {dash}         Security Updates for the Robot Operating System
 ros-updates      no        {dash}         All Updates for the Robot Operating System
 {notices}{features}
@@ -132,6 +136,7 @@ Technical support level: n/a
 # Omit beta services from status
 ATTACHED_STATUS = """\
 SERVICE          ENTITLED  STATUS    DESCRIPTION
+esm-apps         no        {dash}         Expanded Security Maintenance for Applications
 esm-infra        no        {dash}         Expanded Security Maintenance for Infrastructure
 livepatch        no        {dash}         Canonical Livepatch service
 {notices}{features}
@@ -143,7 +148,6 @@ Enable services with: pro enable <service>
 Technical support level: n/a
 """  # noqa: E501
 
-BETA_SVC_NAMES = ["esm-apps", "realtime-kernel", "ros", "ros-updates"]
 
 SERVICES_JSON_ALL = [
     {
@@ -155,6 +159,7 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "yes",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": "Expanded Security Maintenance for Infrastructure",
@@ -165,6 +170,7 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "yes",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": "NIST-certified core packages",
@@ -175,6 +181,7 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "no",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": (
@@ -187,6 +194,7 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "no",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": "Canonical Livepatch service",
@@ -197,9 +205,10 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "yes",
         "blocked_by": [],
+        "warning": None,
     },
     {
-        "description": "Beta-version Ubuntu Kernel with PREEMPT_RT patches",
+        "description": "Ubuntu kernel with PREEMPT_RT patches integrated",
         "description_override": None,
         "entitled": "no",
         "name": "realtime-kernel",
@@ -207,6 +216,7 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "no",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": "Security Updates for the Robot Operating System",
@@ -217,6 +227,7 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "no",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": "All Updates for the Robot Operating System",
@@ -227,10 +238,22 @@ SERVICES_JSON_ALL = [
         "status_details": "",
         "available": "no",
         "blocked_by": [],
+        "warning": None,
     },
 ]
 
 SERVICES_JSON = [
+    {
+        "description": "Expanded Security Maintenance for Applications",
+        "description_override": None,
+        "entitled": "no",
+        "name": "esm-apps",
+        "status": "—",
+        "status_details": "",
+        "available": "yes",
+        "blocked_by": [],
+        "warning": None,
+    },
     {
         "description": "Expanded Security Maintenance for Infrastructure",
         "description_override": None,
@@ -240,6 +263,7 @@ SERVICES_JSON = [
         "status_details": "",
         "available": "yes",
         "blocked_by": [],
+        "warning": None,
     },
     {
         "description": "Canonical Livepatch service",
@@ -250,6 +274,7 @@ SERVICES_JSON = [
         "status_details": "",
         "available": "yes",
         "blocked_by": [],
+        "warning": None,
     },
 ]
 
@@ -303,8 +328,8 @@ Flags:
 )
 
 
+@mock.patch("uaclient.livepatch.on_supported_kernel", return_value=None)
 @mock.patch("uaclient.cli.contract.is_contract_changed", return_value=False)
-@mock.patch("uaclient.files.NoticeFile.remove")
 @mock.patch("uaclient.system.should_reboot", return_value=False)
 @mock.patch(
     "uaclient.status.get_available_resources",
@@ -314,16 +339,14 @@ Flags:
     "uaclient.status.get_contract_information",
     return_value=RESPONSE_CONTRACT_INFO,
 )
-@mock.patch(M_PATH + "os.getuid", return_value=0)
 class TestActionStatus:
     def test_status_help(
         self,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_available_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         capsys,
         FakeConfig,
     ):
@@ -343,8 +366,11 @@ class TestActionStatus:
         (
             ([], ""),
             (
-                [["a", "adesc"], ["b2", "bdesc"]],
-                "\nNOTICES\n a: adesc\nb2: bdesc\n",
+                [
+                    [FakeNotice.a, "adesc"],
+                    [FakeNotice.b, "bdesc"],
+                ],
+                "\nNOTICES\nadesc\nbdesc\n",
             ),
         ),
     )
@@ -361,24 +387,25 @@ class TestActionStatus:
     @mock.patch("uaclient.status.format_expires", return_value="formatteddate")
     def test_attached(
         self,
-        _m_getuid,
+        _m_format_expires,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
-        _m_format_expires,
-        notices,
-        notice_status,
+        _m_on_supported_kernel,
         features,
         feature_status,
+        notices,
+        notice_status,
         use_all,
         capsys,
         FakeConfig,
     ):
         """Check that root and non-root will emit attached status"""
         cfg = FakeConfig.for_attached_machine()
-        cfg.write_cache("notices", notices)
+        mock_notice = NoticesManager()
+        for notice in notices:
+            mock_notice.add(notice[0], notice[1])
         with mock.patch(
             "uaclient.config.UAConfig.features",
             new_callable=mock.PropertyMock,
@@ -419,12 +446,11 @@ class TestActionStatus:
     )
     def test_unattached(
         self,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         use_all,
         capsys,
         FakeConfig,
@@ -447,12 +473,11 @@ class TestActionStatus:
     )
     def test_simulated(
         self,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         use_all,
         capsys,
         FakeConfig,
@@ -475,23 +500,24 @@ class TestActionStatus:
         m_sleep,
         _m_subp,
         _m_get_version,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         capsys,
         FakeConfig,
     ):
         """Check that --wait will will block and poll until lock released."""
         cfg = FakeConfig()
+        mock_notice = NoticesManager()
         lock_file = cfg.data_path("lock")
         cfg.write_cache("lock", "123:pro auto-attach")
 
         def fake_sleep(seconds):
             if m_sleep.call_count == 3:
                 os.unlink(lock_file)
+                mock_notice.remove(Notice.OPERATION_IN_PROGRESS)
 
         m_sleep.side_effect = fake_sleep
 
@@ -526,12 +552,11 @@ class TestActionStatus:
     )
     def test_unattached_formats(
         self,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         environ,
         format_type,
         event_logger_mode,
@@ -565,6 +590,7 @@ class TestActionStatus:
             {
                 "name": service["name"],
                 "description": service["description"],
+                "description_override": service["description_override"],
                 "available": service["available"],
             }
             for service in services
@@ -601,7 +627,10 @@ class TestActionStatus:
                 "external_account_ids": [],
             },
             "config_path": None,
-            "config": {"data_dir": mock.ANY},
+            "config": {
+                "data_dir": mock.ANY,
+                "ua_config": mock.ANY,
+            },
             "simulated": False,
             "errors": [],
             "warnings": [],
@@ -611,7 +640,7 @@ class TestActionStatus:
         if format_type == "json":
             assert expected == json.loads(capsys.readouterr()[0])
         else:
-            assert expected == yaml.safe_load(capsys.readouterr()[0])
+            assert expected == safe_load(capsys.readouterr()[0])
 
     @pytest.mark.parametrize(
         "format_type,event_logger_mode",
@@ -632,12 +661,11 @@ class TestActionStatus:
     @pytest.mark.parametrize("use_all", (True, False))
     def test_attached_formats(
         self,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         use_all,
         environ,
         format_type,
@@ -723,7 +751,10 @@ class TestActionStatus:
                 "external_account_ids": [{"IDs": ["id1"], "origin": "AWS"}],
             },
             "config_path": None,
-            "config": {"data_dir": mock.ANY},
+            "config": {
+                "data_dir": mock.ANY,
+                "ua_config": mock.ANY,
+            },
             "simulated": False,
             "errors": [],
             "warnings": [],
@@ -733,7 +764,7 @@ class TestActionStatus:
         if format_type == "json":
             assert expected == json.loads(capsys.readouterr()[0])
         else:
-            yaml_output = yaml.safe_load(capsys.readouterr()[0])
+            yaml_output = safe_load(capsys.readouterr()[0])
 
             # On earlier versions of pyyaml, we don't add the timezone
             # info when converting a date string into a datetime object.
@@ -762,12 +793,11 @@ class TestActionStatus:
     @pytest.mark.parametrize("use_all", (True, False))
     def test_simulated_formats(
         self,
-        _m_getuid,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         use_all,
         format_type,
         event_logger_mode,
@@ -787,15 +817,7 @@ class TestActionStatus:
         ), mock.patch.object(event, "_command", "status"):
             assert 0 == action_status(args, cfg=cfg)
 
-        beta_services = [
-            {
-                "auto_enabled": "no",
-                "available": "no",
-                "description": "Beta-version Ubuntu Kernel with PREEMPT_RT"
-                " patches",
-                "entitled": "no",
-                "name": "realtime-kernel",
-            },
+        services = [
             {
                 "auto_enabled": "yes",
                 "available": "yes",
@@ -803,24 +825,6 @@ class TestActionStatus:
                 "entitled": "no",
                 "name": "esm-apps",
             },
-            {
-                "auto_enabled": "no",
-                "available": "no",
-                "description": "Security Updates for the Robot Operating"
-                " System",
-                "entitled": "no",
-                "name": "ros",
-            },
-            {
-                "auto_enabled": "no",
-                "available": "no",
-                "description": "All Updates for the Robot Operating System",
-                "entitled": "no",
-                "name": "ros-updates",
-            },
-        ]
-
-        services = [
             {
                 "auto_enabled": "yes",
                 "available": "yes",
@@ -850,11 +854,33 @@ class TestActionStatus:
                 "entitled": "yes",
                 "name": "livepatch",
             },
+            {
+                "auto_enabled": "no",
+                "available": "no",
+                "description": "Ubuntu kernel with PREEMPT_RT patches"
+                " integrated",
+                "entitled": "no",
+                "name": "realtime-kernel",
+            },
+            {
+                "auto_enabled": "no",
+                "available": "no",
+                "description": (
+                    "Security Updates for the Robot Operating System"
+                ),
+                "entitled": "no",
+                "name": "ros",
+            },
+            {
+                "auto_enabled": "no",
+                "available": "no",
+                "description": "All Updates for the Robot Operating System",
+                "entitled": "no",
+                "name": "ros-updates",
+            },
         ]
 
-        expected_services = sorted(
-            services + beta_services, key=lambda x: x["name"]
-        )
+        expected_services = sorted(services, key=lambda x: x["name"])
         if not use_all:
             expected_services = [
                 service
@@ -871,13 +897,13 @@ class TestActionStatus:
             "features": {},
             "notices": [],
             "account": {
-                "created_at": "2019-06-14T06:45:50Z",
+                "created_at": util.parse_rfc3339_date("2019-06-14T06:45:50Z"),
                 "external_account_ids": [],
                 "id": "some_id",
                 "name": "Name",
             },
             "contract": {
-                "created_at": "2021-05-21T20:00:53Z",
+                "created_at": util.parse_rfc3339_date("2021-05-21T20:00:53Z"),
                 "id": "some_id",
                 "name": "Name",
                 "products": ["uai-essential-virtual"],
@@ -886,31 +912,42 @@ class TestActionStatus:
             "environment_vars": [],
             "execution_status": "inactive",
             "execution_details": "No Ubuntu Pro operations are running",
-            "expires": "9999-12-31T00:00:00Z",
+            "expires": util.parse_rfc3339_date("9999-12-31T00:00:00Z"),
             "effective": None,
             "services": expected_services,
             "simulated": True,
             "version": mock.ANY,
             "config_path": None,
-            "config": {"data_dir": mock.ANY},
+            "config": {
+                "data_dir": mock.ANY,
+                "ua_config": mock.ANY,
+            },
             "errors": [],
             "warnings": [],
             "result": "success",
         }
 
         if format_type == "json":
-            assert expected == json.loads(capsys.readouterr()[0])
+            assert expected == json.loads(
+                capsys.readouterr()[0], cls=util.DatetimeAwareJSONDecoder
+            )
         else:
-            assert expected == yaml.safe_load(capsys.readouterr()[0])
+            expected["account"]["created_at"] = safe_load(
+                "2019-06-14 06:45:50+00:00"
+            )
+            expected["contract"]["created_at"] = safe_load(
+                "2021-05-21 20:00:53+00:00"
+            )
+            expected["expires"] = safe_load("9999-12-31 00:00:00+00:00")
+            assert expected == safe_load(capsys.readouterr()[0])
 
     def test_error_on_connectivity_errors(
         self,
-        _m_getuid,
         _m_get_contract_information,
         m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         FakeConfig,
     ):
         """Raise UrlError on connectivity issues"""
@@ -933,13 +970,12 @@ class TestActionStatus:
     @mock.patch("uaclient.status.format_expires", return_value="formatteddate")
     def test_unicode_dash_replacement_when_unprintable(
         self,
-        _m_getuid,
+        _m_format_expires,
         _m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
-        _m_format_expires,
+        _m_on_supported_kernel,
         encoding,
         expected_dash,
         use_all,
@@ -949,11 +985,12 @@ class TestActionStatus:
         # encoding accurately in older versions of pytest
         underlying_stdout = io.BytesIO()
         fake_stdout = io.TextIOWrapper(underlying_stdout, encoding=encoding)
+        cfg = FakeConfig.for_attached_machine()
 
         with mock.patch("sys.stdout", fake_stdout):
             action_status(
                 mock.MagicMock(all=use_all, simulate_with_token=None),
-                cfg=FakeConfig.for_attached_machine(),
+                cfg=cfg,
             )
 
         fake_stdout.flush()  # Make sure all output is in underlying_stdout
@@ -994,12 +1031,11 @@ class TestActionStatus:
     )
     def test_errors_are_raised_appropriately(
         self,
-        _m_getuid,
         m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         exception_to_throw,
         exception_type,
         exception_message,
@@ -1029,13 +1065,13 @@ class TestActionStatus:
                 "expired_token",
                 'Contract "some_id" expired on December 31, 2019',
                 "effectiveTo",
-                "2019-12-31T00:00:00Z",
+                util.parse_rfc3339_date("2019-12-31T00:00:00Z"),
             ),
             (
                 "token_not_valid_yet",
                 'Contract "some_id" is not effective until December 31, 9999',
                 "effectiveFrom",
-                "9999-12-31T00:00:00Z",
+                util.parse_rfc3339_date("9999-12-31T00:00:00Z"),
             ),
         ),
     )
@@ -1045,12 +1081,11 @@ class TestActionStatus:
     )
     def test_errors_for_token_dates(
         self,
-        _m_getuid,
         m_get_contract_information,
         _m_get_avail_resources,
         _m_should_reboot,
-        _m_remove_notice,
         _m_contract_changed,
+        _m_on_supported_kernel,
         format_type,
         event_logger_mode,
         token_to_use,
@@ -1071,7 +1106,6 @@ class TestActionStatus:
         m_get_contract_information.side_effect = contract_info_side_effect
 
         cfg = FakeConfig()
-
         args = mock.MagicMock(
             format=format_type, all=False, simulate_with_token=token_to_use
         )
@@ -1084,56 +1118,10 @@ class TestActionStatus:
         if format_type == "json":
             output = json.loads(capsys.readouterr()[0])
         else:
-            output = yaml.safe_load(capsys.readouterr()[0])
+            output = safe_load(capsys.readouterr()[0])
 
         assert output["errors"][0]["message"] == warning_message
 
-    @pytest.mark.parametrize(
-        "contract_changed,is_attached",
-        (
-            (False, True),
-            (True, False),
-            (True, True),
-            (False, False),
-        ),
-    )
-    @mock.patch("uaclient.files.NoticeFile.add")
-    def test_is_contract_changed(
-        self,
-        m_add_notice,
-        _m_getuid,
-        _m_get_contract_information,
-        _m_get_available_resources,
-        _m_should_reboot,
-        _m_remove_notice,
-        _m_contract_changed,
-        contract_changed,
-        is_attached,
-        capsys,
-        FakeConfig,
-    ):
-        _m_contract_changed.return_value = contract_changed
-        if is_attached:
-            cfg = FakeConfig().for_attached_machine()
-        else:
-            cfg = FakeConfig()
-
-        action_status(
-            mock.MagicMock(all=False, simulate_with_token=None), cfg=cfg
-        )
-
-        if is_attached:
-            if contract_changed:
-                assert [
-                    mock.call("", messages.NOTICE_REFRESH_CONTRACT_WARNING)
-                ] == m_add_notice.call_args_list
-            else:
-                assert [
-                    mock.call("", messages.NOTICE_REFRESH_CONTRACT_WARNING)
-                ] not in m_add_notice.call_args_list
-        else:
-            assert _m_contract_changed.call_count == 0
-
 
 class TestStatusParser:
     @mock.patch(M_PATH + "contract.get_available_resources")
