@@ -8,7 +8,6 @@ import stat
 
 import mock
 import pytest
-import yaml
 
 from uaclient import apt, exceptions, messages, util
 from uaclient.config import (
@@ -19,10 +18,14 @@ from uaclient.config import (
     get_config_path,
     parse_config,
 )
+from uaclient.conftest import FakeNotice
 from uaclient.defaults import DEFAULT_CONFIG_FILE
 from uaclient.entitlements import valid_services
 from uaclient.entitlements.entitlement_status import ApplicationStatus
+from uaclient.files import notices
+from uaclient.files.notices import NoticesManager
 from uaclient.util import depth_first_merge_overlay_dict
+from uaclient.yaml import safe_dump
 
 KNOWN_DATA_PATHS = (
     ("machine-access-cis", "machine-access-cis.json"),
@@ -71,63 +74,113 @@ class TestNotices:
     @pytest.mark.parametrize(
         "notices,expected",
         (
-            ([], []),
-            ([["a", "a1"]], [["a", "a1"]]),
-            ([["a", "a1"], ["a", "a1"]], [["a", "a1"]]),
+            ([], ()),
+            (
+                [[FakeNotice.a2, "a1"]],
+                ["a1"],
+            ),
+            (
+                [
+                    [FakeNotice.a, "a1"],
+                    [FakeNotice.a2, "a2"],
+                ],
+                [
+                    "a1",
+                    "a2",
+                ],
+            ),
+            (
+                [
+                    [FakeNotice.a, "a1"],
+                    [FakeNotice.a, "a1"],
+                ],
+                [
+                    "a1",
+                ],
+            ),
         ),
     )
     def test_add_notice_avoids_duplicates(
-        self, notices, expected, tmpdir, FakeConfig
+        self,
+        notices,
+        expected,
     ):
-        cfg = FakeConfig()
-        assert None is cfg.notice_file.read()
-        for notice in notices:
-            cfg.notice_file.add(*notice)
+        notice = NoticesManager()
+        assert [] == notice.list()
+        for notice_ in notices:
+            notice.add(*notice_)
         if notices:
-            assert expected == cfg.notice_file.read()
+            assert expected == notice.list()
         else:
-            assert None is cfg.notice_file.read()
+            assert [] == notice.list()
 
     @pytest.mark.parametrize(
-        "notices,expected",
+        "_notices",
         (
-            ([], []),
-            ([["a", "a1"]], [["a", "a1"]]),
-            ([["a", "a1"], ["a", "a1"]], [["a", "a1"]]),
+            ([]),
+            ([[FakeNotice.a]]),
+            (
+                [
+                    [FakeNotice.a],
+                    [FakeNotice.a2],
+                ]
+            ),
         ),
     )
+    @mock.patch("uaclient.util.we_are_currently_root", return_value=False)
     def test_add_notice_fails_as_nonroot(
-        self, notices, expected, tmpdir, FakeConfig
+        self,
+        m_we_are_currently_root,
+        _notices,
     ):
-        cfg = FakeConfig(root_mode=False)
-        assert None is cfg.notice_file.read()
-        for notice in notices:
-            with pytest.raises(exceptions.NonRootUserError):
-                cfg.notice_file.add(*notice)
-        assert None is cfg.notice_file.read()
+        assert [] == notices.list()
+        for notice_ in _notices:
+            notices.add(*notice_)
+        assert [] == notices.list()
 
     @pytest.mark.parametrize(
-        "notices,removes,expected",
+        "notices_,removes,expected",
         (
-            ([], [["a", "a1"]], None),
-            ([["a", "a1"]], [["a", "a1"]], None),
-            ([["a", "a1"], ["a", "a2"]], [["a", "a1"]], [["a", "a2"]]),
+            ([], [FakeNotice.a], []),
             (
-                [["a", "a1"], ["a", "a2"], ["b", "b2"]],
-                [["a", ".*"]],
-                [["b", "b2"]],
+                [[FakeNotice.a2]],
+                [FakeNotice.a2],
+                [],
+            ),
+            (
+                [
+                    [FakeNotice.a],
+                    [FakeNotice.a2],
+                ],
+                [FakeNotice.a],
+                ["notice_a2"],
+            ),
+            (
+                [
+                    [FakeNotice.a],
+                    [FakeNotice.a2],
+                    [FakeNotice.b],
+                ],
+                [
+                    FakeNotice.a,
+                    FakeNotice.a2,
+                ],
+                ["notice_b"],
             ),
         ),
     )
     def test_remove_notice_removes_matching(
-        self, notices, removes, expected, tmpdir, FakeConfig
+        self,
+        notices_,
+        removes,
+        expected,
     ):
-        cfg = FakeConfig()
-        for notice in notices:
-            cfg.notice_file.add(*notice)
-        for label, descr in removes:
-            cfg.notice_file.remove(label, descr)
-        assert expected == cfg.notice_file.read()
+
+        for notice_ in notices_:
+            notices.add(*notice_)
+        for label in removes:
+            notices.remove(label)
+        assert expected == notices.list()
 
 
 class TestEntitlements:
@@ -158,9 +211,8 @@ class TestEntitlements:
         }
         assert expected == cfg.machine_token_file.entitlements
 
-    @mock.patch("os.getuid", return_value=0)
     def test_entitlements_uses_resource_token_from_machine_token(
-        self, tmpdir, FakeConfig, all_resources_available
+        self, FakeConfig, all_resources_available
     ):
         """Include entitlement-specific resourceTokens from machine_token"""
         cfg = FakeConfig()
@@ -194,16 +246,16 @@ class TestEntitlements:
 
 
 class TestAccounts:
-    def test_accounts_returns_empty_list_when_no_cached_account_value(
+    def test_accounts_returns_none_when_no_cached_account_value(
         self, tmpdir, FakeConfig, all_resources_available
     ):
         """Config.accounts property returns an empty list when no cache."""
         cfg = FakeConfig()
 
-        assert [] == cfg.machine_token_file.accounts
+        assert cfg.machine_token_file.account is None
 
     @pytest.mark.usefixtures("all_resources_available")
-    def test_accounts_extracts_accounts_key_from_machine_token_cache(
+    def test_accounts_extracts_account_key_from_machine_token_cache(
         self, all_resources_available, tmpdir, FakeConfig
     ):
         """Use machine_token cached accountInfo when no accounts cache."""
@@ -217,7 +269,7 @@ class TestAccounts:
             },
         )
 
-        assert [accountInfo] == cfg.machine_token_file.accounts
+        assert accountInfo == cfg.machine_token_file.account
 
 
 class TestDataPath:
@@ -291,88 +343,36 @@ settings_overrides:
 timer_log_file: /var/log/ubuntu-advantage-timer.log
 """
 
-UA_CFG_DICT = {
-    "ua_config": {
-        "apt_http_proxy": None,
-        "apt_https_proxy": None,
-        "global_apt_http_proxy": None,
-        "global_apt_https_proxy": None,
-        "ua_apt_http_proxy": None,
-        "ua_apt_https_proxy": None,
-        "http_proxy": None,
-        "https_proxy": None,
-        "update_messaging_timer": None,
-        "update_status_timer": None,
-        "metering_timer": None,
-    }
+USER_CFG_DICT = {
+    "apt_http_proxy": None,
+    "apt_https_proxy": None,
+    "apt_news": True,
+    "apt_news_url": "https://motd.ubuntu.com/aptnews.json",
+    "global_apt_http_proxy": None,
+    "global_apt_https_proxy": None,
+    "ua_apt_http_proxy": None,
+    "ua_apt_https_proxy": None,
+    "http_proxy": None,
+    "https_proxy": None,
+    "update_messaging_timer": 21600,
+    "metering_timer": 14400,
 }
 
 
-class TestUAConfigKeys:
+class TestUserConfigKeys:
     @pytest.mark.parametrize("attr_name", UA_CONFIGURABLE_KEYS)
-    @mock.patch("uaclient.config.UAConfig.write_cfg")
-    def test_ua_configurable_keys_set_ua_config_dict(
-        self, write_cfg, attr_name, tmpdir, FakeConfig
+    @mock.patch("uaclient.config.state_files.user_config_file.write")
+    def test_user_configurable_keys_set_user_config(
+        self, write, attr_name, tmpdir, FakeConfig
     ):
         """Getters and settings are available fo UA_CONFIGURABLE_KEYS."""
         cfg = FakeConfig()
-        assert None is getattr(cfg, attr_name, None)
+        assert USER_CFG_DICT[attr_name] == getattr(cfg, attr_name, None)
         cfg_non_members = ("apt_http_proxy", "apt_https_proxy")
         if attr_name not in cfg_non_members:
             setattr(cfg, attr_name, attr_name + "value")
             assert attr_name + "value" == getattr(cfg, attr_name)
-            assert attr_name + "value" == cfg.cfg["ua_config"][attr_name]
-
-
-class TestWriteCfg:
-    @pytest.mark.parametrize(
-        "orig_content, expected",
-        (
-            (
-                CFG_BASE_CONTENT,
-                CFG_BASE_CONTENT
-                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
-            ),
-            (  # Yaml output is sorted alphabetically by key
-                "\n".join(sorted(CFG_BASE_CONTENT.splitlines(), reverse=True)),
-                CFG_BASE_CONTENT
-                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
-            ),
-            # Any custom comments or unrecognized config keys are dropped
-            (
-                "unknown-keys-not-preserved: true\n# user comments are lost"
-                + CFG_BASE_CONTENT,
-                CFG_BASE_CONTENT
-                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
-            ),
-            # All features/settings_overrides ordered after ua_config
-            (
-                CFG_BASE_CONTENT
-                + "features:\n new: 2\n extra_security_params:\n  hide: true\n"
-                " show_beta: true\nsettings_overrides:\n d: 2\n c: 1\n",
-                CFG_FEATURES_CONTENT
-                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
-            ),
-            (
-                "settings_overrides:\n c: 1\n d: 2\nfeatures:\n"
-                " show_beta: true\n new: 2\n extra_security_params:\n"
-                "  hide: true\nsettings_overrides:\n d: 2\n c: 1\n"
-                + CFG_BASE_CONTENT,
-                CFG_FEATURES_CONTENT
-                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
-            ),
-        ),
-    )
-    def test_write_cfg_reads_cfg_andpersists_structured_content_to_config_path(
-        self, orig_content, expected, tmpdir, FakeConfig
-    ):
-        """write_cfg writes structured, ordered config YAML to config_path."""
-        orig_conf = tmpdir.join("orig_uaclient.conf")
-        orig_conf.write(orig_content)
-        cfg = FakeConfig(cfg_overrides=parse_config(orig_conf.strpath)[0])
-        out_conf = tmpdir.join("uaclient.conf")
-        cfg.write_cfg(out_conf.strpath)
-        assert expected == out_conf.read()
+            assert attr_name + "value" == getattr(cfg.user_config, attr_name)
 
 
 class TestWriteCache:
@@ -861,8 +861,8 @@ class TestProcessConfig:
         ],
     )
     @mock.patch("uaclient.util.validate_proxy")
-    @mock.patch("uaclient.entitlements.livepatch.get_config_option_value")
-    @mock.patch("uaclient.entitlements.livepatch.configure_livepatch_proxy")
+    @mock.patch("uaclient.livepatch.get_config_option_value")
+    @mock.patch("uaclient.livepatch.configure_livepatch_proxy")
     @mock.patch(
         "uaclient.entitlements.livepatch.LivepatchEntitlement.application_status"  # noqa: E501
     )
@@ -870,10 +870,10 @@ class TestProcessConfig:
     @mock.patch("uaclient.snap.configure_snap_proxy")
     @mock.patch("uaclient.snap.is_installed")
     @mock.patch("uaclient.apt.setup_apt_proxy")
-    @mock.patch("uaclient.config.UAConfig.write_cfg")
+    @mock.patch("uaclient.config.state_files.user_config_file.write")
     def test_process_config(
         self,
-        m_write_cfg,
+        m_write,
         m_apt_configure_proxy,
         m_snap_is_installed,
         m_snap_configure_proxy,
@@ -912,24 +912,17 @@ class TestProcessConfig:
             livepatch_http_val,
             livepatch_https_val,
         ]
-        cfg = FakeConfig(
-            {
-                "ua_config": {
-                    "apt_http_proxy": apt_http,
-                    "apt_https_proxy": apt_https,
-                    "global_apt_https_proxy": global_https,
-                    "global_apt_http_proxy": global_http,
-                    "ua_apt_https_proxy": ua_https,
-                    "ua_apt_http_proxy": ua_http,
-                    "http_proxy": http_proxy,
-                    "https_proxy": https_proxy,
-                    "update_messaging_timer": 21600,
-                    "update_status_timer": 43200,
-                    "metering_timer": 0,
-                },
-                "data_dir": tmpdir.strpath,
-            }
-        )
+        cfg = FakeConfig({"data_dir": tmpdir.strpath})
+        cfg.user_config.apt_http_proxy = apt_http
+        cfg.user_config.apt_https_proxy = apt_https
+        cfg.user_config.global_apt_https_proxy = global_https
+        cfg.user_config.global_apt_http_proxy = global_http
+        cfg.user_config.ua_apt_https_proxy = ua_https
+        cfg.user_config.ua_apt_http_proxy = ua_http
+        cfg.user_config.http_proxy = http_proxy
+        cfg.user_config.https_proxy = https_proxy
+        cfg.user_config.update_messaging_timer = 21600
+        cfg.user_config.metering_timer = 0
 
         if global_https is None and apt_https is not None:
             global_https = apt_https
@@ -1018,14 +1011,8 @@ class TestProcessConfig:
             assert "" == err
 
     def test_process_config_errors_for_wrong_timers(self, FakeConfig):
-        cfg = FakeConfig(
-            {
-                "ua_config": {
-                    "update_messaging_timer": "wrong",
-                    "update_status_timer": 43200,
-                }
-            }
-        )
+        cfg = FakeConfig()
+        cfg.user_config.update_messaging_timer = "wrong"
 
         with pytest.raises(
             exceptions.UserFacingError,
@@ -1056,7 +1043,7 @@ class TestParseConfig:
             "log_file": "/var/log/ubuntu-advantage.log",
             "timer_log_file": "/var/log/ubuntu-advantage-timer.log",
             "daemon_log_file": "/var/log/ubuntu-advantage-daemon.log",  # noqa: E501
-            "log_level": "INFO",
+            "log_level": "debug",
         }
         assert expected_default_config == config
 
@@ -1074,7 +1061,7 @@ class TestParseConfig:
         self, config_dict, expected_invalid_keys, tmpdir
     ):
         config_file = tmpdir.join("uaclient.conf")
-        config_file.write(yaml.dump(config_dict))
+        config_file.write(safe_dump(config_dict))
         env_vars = {"UA_CONFIG_FILE": config_file.strpath}
         with mock.patch.dict("uaclient.config.os.environ", values=env_vars):
             cfg, invalid_keys = parse_config(config_file.strpath)
@@ -1315,10 +1302,7 @@ class TestMachineTokenOverlay:
         assert expected == cfg.machine_token
 
     @mock.patch("uaclient.files.MachineTokenFile.read")
-    @mock.patch("os.getuid", return_value=0)
-    def test_machine_token_without_overlay(
-        self, _m_getuid, m_token_read, FakeConfig
-    ):
+    def test_machine_token_without_overlay(self, m_token_read, FakeConfig):
         user_cfg = {}
         m_token_read.return_value = self.machine_token_dict
         cfg = FakeConfig(cfg_overrides=user_cfg)
@@ -1417,3 +1401,29 @@ class TestGetConfigPath:
         with mock.patch.dict("uaclient.config.os.environ", values={}):
             assert DEFAULT_CONFIG_FILE == get_config_path()
             assert _m_exists.call_count == 1
+
+
+class TestCheckLockInfo:
+    @pytest.mark.parametrize("lock_content", ((""), ("corrupted")))
+    @mock.patch("os.path.exists", return_value=True)
+    @mock.patch("uaclient.system.load_file")
+    def test_raise_exception_for_corrupted_lock(
+        self,
+        m_load_file,
+        _m_path_exists,
+        lock_content,
+        FakeConfig,
+    ):
+        cfg = FakeConfig()
+        m_load_file.return_value = lock_content
+
+        expected_msg = messages.INVALID_LOCK_FILE.format(
+            lock_file_path=cfg.data_dir + "/lock"
+        )
+
+        with pytest.raises(exceptions.InvalidLockFile) as exc_info:
+            cfg.check_lock_info()
+
+        assert expected_msg.msg == exc_info.value.msg
+        assert m_load_file.call_count == 1
+        assert _m_path_exists.call_count == 1
