@@ -356,6 +356,13 @@ class ExampleDataObject(DataObject):
         self.dtlist_opt = dtlist_opt
 
 
+class ExampleObjectWithDictKey(DataObject):
+    fields = [Field("string", StringDataValue, dict_key="S-t/R*i_n#g")]
+
+    def __init__(self, *, string: str):
+        self.string = string
+
+
 example_data_object_dict_no_optionals = {
     "string": "string",
     "integer": 1,
@@ -683,6 +690,40 @@ class TestDataObject:
             ExampleDataObject.from_value(val)
         assert e.value.msg == error.msg
 
+    def test_dict_key(self):
+        d = {"S-t/R*i_n#g": "something"}
+        do = ExampleObjectWithDictKey.from_dict(d)
+        assert do.string == "something"
+        assert d == do.to_dict()
+
+    def test_optional_type_errors_become_null(self):
+        result = ExampleDataObject.from_dict(
+            {
+                **example_data_object_dict_with_optionals,
+                "string_opt": 4,
+                "integer_opt": {},
+                "obj_opt": [],
+                "stringlist_opt": [4],
+                "integerlist_opt": ["hello"],
+                "objlist_opt": [[]],
+                "enum_opt": "nope",
+                "enum_opt_list": "nope",
+                "dt_opt": 1234,
+                "dtlist_opt": 1234,
+            },
+            optional_type_errors_become_null=True,
+        )
+        assert result.string_opt is None
+        assert result.integer_opt is None
+        assert result.obj_opt is None
+        assert result.stringlist_opt is None
+        assert result.integerlist_opt is None
+        assert result.objlist_opt is None
+        assert result.enum_opt is None
+        assert result.enum_opt_list is None
+        assert result.dt_opt is None
+        assert result.dtlist_opt is None
+
     @pytest.mark.parametrize(
         "d",
         (
