@@ -23,6 +23,7 @@ from uaclient.entitlements.entitlement_status import (
 from uaclient.entitlements.fips import FIPSEntitlement
 from uaclient.entitlements.ros import ROSEntitlement
 from uaclient.entitlements.tests.test_base import ConcreteTestEntitlement
+from uaclient.files.notices import Notice, NoticesManager
 from uaclient.status import (
     DEFAULT_STATUS,
     TxtColor,
@@ -129,6 +130,7 @@ class TestColorizeCommands:
         assert colorize_commands(commands) == expected
 
 
+@mock.patch("uaclient.livepatch.on_supported_kernel", return_value=None)
 class TestFormatTabular:
     @pytest.mark.parametrize(
         "support_level,expected_colour,istty",
@@ -150,6 +152,7 @@ class TestFormatTabular:
     def test_support_colouring(
         self,
         m_isatty,
+        m_on_supported_kernel,
         support_level,
         expected_colour,
         istty,
@@ -168,7 +171,9 @@ class TestFormatTabular:
         assert expected_string in tabular_output
 
     @pytest.mark.parametrize("origin", ["free", "not-free"])
-    def test_header_alignment(self, origin, status_dict_attached):
+    def test_header_alignment(
+        self, m_on_supported_kernel, origin, status_dict_attached
+    ):
         status_dict_attached["origin"] = origin
         tabular_output = format_tabular(status_dict_attached)
         colon_idx = None
@@ -191,7 +196,11 @@ class TestFormatTabular:
         ],
     )
     def test_correct_header_keys_included(
-        self, origin, expected_headers, status_dict_attached
+        self,
+        m_on_supported_kernel,
+        origin,
+        expected_headers,
+        status_dict_attached,
     ):
         status_dict_attached["origin"] = origin
 
@@ -209,7 +218,9 @@ class TestFormatTabular:
         ]
         assert list(expected_headers) == headers
 
-    def test_correct_unattached_column_alignment(self, status_dict_unattached):
+    def test_correct_unattached_column_alignment(
+        self, m_on_supported_kernel, status_dict_unattached
+    ):
         tabular_output = format_tabular(status_dict_unattached)
         [header, eal_service_line] = [
             line
@@ -224,7 +235,11 @@ class TestFormatTabular:
 
     @pytest.mark.parametrize("attached", [True, False])
     def test_no_leading_newline(
-        self, attached, status_dict_attached, status_dict_unattached
+        self,
+        m_on_supported_kernel,
+        attached,
+        status_dict_attached,
+        status_dict_unattached,
     ):
         if attached:
             status_dict = status_dict_attached
@@ -242,7 +257,12 @@ class TestFormatTabular:
         ),
     )
     def test_custom_descr(
-        self, description_override, uf_status, uf_descr, status_dict_attached
+        self,
+        m_on_supported_kernel,
+        description_override,
+        uf_status,
+        uf_descr,
+        status_dict_attached,
     ):
         """Services can provide a custom call to action if present."""
         default_descr = "Common Criteria EAL2 default descr"
@@ -272,7 +292,8 @@ def ros_desc(FakeConfig):
     return entitlement_factory(cfg=FakeConfig(), name="ros").description
 
 
-@mock.patch("uaclient.files.NoticeFile.remove")
+@mock.patch("uaclient.livepatch.on_supported_kernel", return_value=None)
+@mock.patch("uaclient.files.notices.NoticesManager.remove")
 @mock.patch("uaclient.system.should_reboot", return_value=False)
 class TestStatus:
     def check_beta(self, cls, show_all, uacfg=None, status=""):
@@ -298,7 +319,8 @@ class TestStatus:
         self,
         m_get_available_resources,
         _m_should_reboot,
-        m_remove_notice,
+        _m_remove_notice,
+        m_on_supported_kernel,
         ros_desc,
         esm_desc,
         show_all,
@@ -311,11 +333,13 @@ class TestStatus:
                     "available": "yes",
                     "name": "esm-infra",
                     "description": esm_desc,
+                    "description_override": None,
                 },
                 {
                     "available": "no",
                     "name": "ros",
                     "description": ros_desc,
+                    "description_override": None,
                 },
             ]
         else:
@@ -324,6 +348,7 @@ class TestStatus:
                     "available": "yes",
                     "name": "esm-infra",
                     "description": esm_desc,
+                    "description_override": None,
                 }
             ]
         cfg = FakeConfig()
@@ -340,17 +365,6 @@ class TestStatus:
             m_get_cfg_status.return_value = DEFAULT_CFG_STATUS
             assert expected == status.status(cfg=cfg, show_all=show_all)
 
-            expected_calls = [
-                mock.call(
-                    "",
-                    messages.ENABLE_REBOOT_REQUIRED_TMPL.format(
-                        operation="fix operation"
-                    ),
-                )
-            ]
-
-            assert expected_calls == m_remove_notice.call_args_list
-
     @pytest.mark.parametrize(
         "features_override", ((None), ({"allow_beta": True}))
     )
@@ -394,6 +408,7 @@ class TestStatus:
         _m_livepatch_status,
         _m_should_reboot,
         _m_remove_notice,
+        m_on_supported_kernel,
         avail_res,
         entitled_res,
         uf_entitled,
@@ -412,15 +427,47 @@ class TestStatus:
                 "accountInfo": {
                     "id": "acct-1",
                     "name": "test_account",
-                    "createdAt": "2019-06-14T06:45:50Z",
+                    "createdAt": datetime.datetime(
+                        2019,
+                        6,
+                        14,
+                        6,
+                        45,
+                        50,
+                        tzinfo=datetime.timezone.utc,
+                    ),
                     "externalAccountIDs": [{"IDs": ["id1"], "origin": "AWS"}],
                 },
                 "contractInfo": {
                     "id": "cid",
                     "name": "test_contract",
-                    "createdAt": "2020-05-08T19:02:26Z",
-                    "effectiveFrom": "2000-05-08T19:02:26Z",
-                    "effectiveTo": "2040-05-08T19:02:26Z",
+                    "createdAt": datetime.datetime(
+                        2020,
+                        5,
+                        8,
+                        19,
+                        2,
+                        26,
+                        tzinfo=datetime.timezone.utc,
+                    ),
+                    "effectiveFrom": datetime.datetime(
+                        2000,
+                        5,
+                        8,
+                        19,
+                        2,
+                        26,
+                        tzinfo=datetime.timezone.utc,
+                    ),
+                    "effectiveTo": datetime.datetime(
+                        2040,
+                        5,
+                        8,
+                        19,
+                        2,
+                        26,
+                        tzinfo=datetime.timezone.utc,
+                    ),
                     "resourceEntitlements": entitled_res,
                     "products": ["free"],
                 },
@@ -461,6 +508,7 @@ class TestStatus:
                 "description_override": None,
                 "available": mock.ANY,
                 "blocked_by": [],
+                "warning": None,
             }
             for cls in ENTITLEMENT_CLASSES
         ]
@@ -514,36 +562,46 @@ class TestStatus:
             m_get_cfg_status.return_value = DEFAULT_CFG_STATUS
             assert expected == status.status(cfg=cfg, show_all=True)
 
+    @mock.patch("uaclient.util.we_are_currently_root")
     @mock.patch("uaclient.status.get_available_resources")
     def test_nonroot_unattached_is_same_as_unattached_root(
         self,
         m_get_available_resources,
+        m_we_are_currently_root,
         _m_should_reboot,
         _m_remove_notice,
+        m_on_supported_kernel,
         FakeConfig,
     ):
         m_get_available_resources.return_value = [
             {"name": "esm-infra", "available": True}
         ]
-        cfg = FakeConfig(root_mode=False)
+        m_we_are_currently_root.return_value = False
+        cfg = FakeConfig()
         nonroot_status = status.status(cfg=cfg)
 
-        cfg = FakeConfig(root_mode=True)
+        m_we_are_currently_root.return_value = True
+        cfg = FakeConfig()
         root_unattached_status = status.status(cfg=cfg)
 
         assert root_unattached_status == nonroot_status
 
+    @mock.patch("uaclient.util.we_are_currently_root")
     @mock.patch("uaclient.status.get_available_resources")
     def test_root_and_non_root_are_same_attached(
         self,
         m_get_available_resources,
+        m_we_are_currently_root,
         _m_should_reboot,
         _m_remove_notice,
+        m_on_supported_kernel,
         FakeConfig,
     ):
+        m_we_are_currently_root.return_value = True
         root_cfg = FakeConfig.for_attached_machine()
         root_status = status.status(cfg=root_cfg)
-        normal_cfg = FakeConfig(root_mode=False)
+        m_we_are_currently_root.return_value = False
+        normal_cfg = FakeConfig.for_attached_machine()
         normal_status = status.status(cfg=normal_cfg)
         assert normal_status == root_status
 
@@ -553,6 +611,7 @@ class TestStatus:
         _m_get_available_resources,
         _m_should_reboot,
         m_remove_notice,
+        m_on_supported_kernel,
         FakeConfig,
     ):
         cfg = FakeConfig()
@@ -562,17 +621,6 @@ class TestStatus:
             os.lstat(cfg.data_path("status-cache")).st_mode
         )
 
-        expected_calls = [
-            mock.call(
-                "",
-                messages.ENABLE_REBOOT_REQUIRED_TMPL.format(
-                    operation="fix operation"
-                ),
-            )
-        ]
-
-        assert expected_calls == m_remove_notice.call_args_list
-
     @pytest.mark.parametrize("show_all", (True, False))
     @pytest.mark.parametrize(
         "features_override", ((None), ({"allow_beta": False}))
@@ -616,7 +664,8 @@ class TestStatus:
         _m_livepatch_status,
         _m_fips_status,
         _m_should_reboot,
-        m_remove_notice,
+        _m_remove_notice,
+        m_on_supported_kernel,
         all_resources_available,
         entitlements,
         features_override,
@@ -646,13 +695,17 @@ class TestStatus:
                 "accountInfo": {
                     "id": "1",
                     "name": "accountname",
-                    "createdAt": "2019-06-14T06:45:50Z",
+                    "createdAt": datetime.datetime(
+                        2019, 6, 14, 6, 45, 50, tzinfo=datetime.timezone.utc
+                    ),
                     "externalAccountIDs": [{"IDs": ["id1"], "origin": "AWS"}],
                 },
                 "contractInfo": {
                     "id": "contract-1",
                     "name": "contractname",
-                    "createdAt": "2020-05-08T19:02:26Z",
+                    "createdAt": datetime.datetime(
+                        2020, 5, 8, 19, 2, 26, tzinfo=datetime.timezone.utc
+                    ),
                     "resourceEntitlements": entitlements,
                     "products": ["free"],
                 },
@@ -662,6 +715,7 @@ class TestStatus:
             account_name="accountname",
             machine_token=token,
         )
+        mock_notice = NoticesManager()
         if features_override:
             cfg.override_features(features_override)
         if not entitlements:
@@ -726,6 +780,7 @@ class TestStatus:
                     "description_override": None,
                     "available": mock.ANY,
                     "blocked_by": [],
+                    "warning": None,
                 }
             )
         with mock.patch(
@@ -734,37 +789,30 @@ class TestStatus:
             m_get_cfg_status.return_value = DEFAULT_CFG_STATUS
             assert expected == status.status(cfg=cfg, show_all=show_all)
 
-        assert len(ENTITLEMENT_CLASSES) - 2 == m_repo_uf_status.call_count
-        assert 1 == m_livepatch_uf_status.call_count
+            assert len(ENTITLEMENT_CLASSES) - 2 == m_repo_uf_status.call_count
+            assert 1 == m_livepatch_uf_status.call_count
 
         expected_calls = [
-            mock.call(
-                "",
-                messages.NOTICE_DAEMON_AUTO_ATTACH_LOCK_HELD.format(
-                    operation=".*"
-                ),
-            ),
-            mock.call("", messages.NOTICE_DAEMON_AUTO_ATTACH_FAILED),
-            mock.call(
-                "",
-                messages.ENABLE_REBOOT_REQUIRED_TMPL.format(
-                    operation="fix operation"
-                ),
-            ),
+            mock.call(Notice.AUTO_ATTACH_RETRY_FULL_NOTICE),
+            mock.call(Notice.AUTO_ATTACH_RETRY_TOTAL_FAILURE),
         ]
 
-        assert expected_calls == m_remove_notice.call_args_list
+        assert expected_calls == mock_notice.remove.call_args_list
 
     @pytest.mark.usefixtures("all_resources_available")
+    @mock.patch("uaclient.util.we_are_currently_root")
     @mock.patch("uaclient.status.get_available_resources")
     def test_expires_handled_appropriately(
         self,
         _m_get_available_resources,
+        m_we_are_currently_root,
         _m_should_reboot,
         _m_remove_notice,
+        m_on_supported_kernel,
         all_resources_available,
         FakeConfig,
     ):
+        m_we_are_currently_root.return_value = True
         token = {
             "availableResources": all_resources_available,
             "machineTokenInfo": {
@@ -773,8 +821,12 @@ class TestStatus:
                 "contractInfo": {
                     "name": "contractname",
                     "id": "contract-1",
-                    "effectiveTo": "2020-07-18T00:00:00Z",
-                    "createdAt": "2020-05-08T19:02:26Z",
+                    "effectiveTo": datetime.datetime(
+                        2020, 7, 18, 0, 0, 0, tzinfo=datetime.timezone.utc
+                    ),
+                    "createdAt": datetime.datetime(
+                        2020, 5, 8, 0, 0, 0, tzinfo=datetime.timezone.utc
+                    ),
                     "resourceEntitlements": [],
                     "products": ["free"],
                 },
@@ -793,7 +845,11 @@ class TestStatus:
 
         # Test that the read from the status cache work properly for non-root
         # users
-        cfg = FakeConfig(root_mode=False)
+        m_we_are_currently_root.return_value = False
+        cfg = FakeConfig.for_attached_machine(
+            account_name="accountname",
+            machine_token=token,
+        )
         assert expected_dt == status.status(cfg=cfg)["expires"]
 
     @mock.patch("uaclient.status.get_available_resources", return_value={})
@@ -802,6 +858,7 @@ class TestStatus:
         _m_get_available_resources,
         _m_should_reboot,
         m_remove_notice,
+        m_on_supported_kernel,
         FakeConfig,
     ):
 
