@@ -133,92 +133,6 @@ class TestGetKernelInfo:
         assert system.get_kernel_info.__wrapped__() == expected
 
 
-class TestGetLscpuArch:
-    @pytest.mark.parametrize(
-        "stdout, expected",
-        (
-            (
-                """\
-Architecture:            x86_64
-  CPU op-mode(s):        32-bit, 64-bit
-  Address sizes:         39 bits physical, 48 bits virtual
-  Byte Order:            Little Endian
-CPU(s):                  8
-  On-line CPU(s) list:   0-7
-""",
-                "x86_64",
-            ),
-            (
-                """\
-Architecture:            aarch64
-""",
-                "aarch64",
-            ),
-            (
-                """\
-CPU(s):                  8
-  On-line CPU(s) list:   0-7
-Architecture:            x86_64
-  CPU op-mode(s):        32-bit, 64-bit
-  Address sizes:         39 bits physical, 48 bits virtual
-  Byte Order:            Little Endian
-""",
-                "x86_64",
-            ),
-            (
-                """\
-  CPU(s):                  8
-    On-line CPU(s) list:   0-7
-
-  Architecture:            x86_64
-    CPU op-mode(s):        32-bit, 64-bit
-    Address sizes:         39 bits physical, 48 bits virtual
-    Byte Order:            Little Endian
-""",
-                "x86_64",
-            ),
-            (
-                """Architecture: x86_64""",
-                "x86_64",
-            ),
-            (
-                """Architecture:x86_64""",
-                "x86_64",
-            ),
-            (
-                """       Architecture:         x86_64         """,
-                "x86_64",
-            ),
-        ),
-    )
-    @mock.patch("uaclient.system.subp")
-    def test_get_lscpu_arch_success(self, m_subp, stdout, expected):
-        m_subp.return_value = (stdout, "")
-        assert system.get_lscpu_arch.__wrapped__() == expected
-        assert m_subp.call_args_list == [mock.call(["lscpu"])]
-
-    @pytest.mark.parametrize(
-        "stdout",
-        (
-            "Architecture            x86_64",
-            "Architectur:            x86_64",
-            "rchitecture:            x86_64",
-            "architecture:            x86_64",
-            ":            x86_64",
-            "Architecture:",
-            "Architecture: ",
-            "Architecture:           ",
-            "",
-        ),
-    )
-    @mock.patch("uaclient.system.subp")
-    def test_get_lscpu_arch_error(self, m_subp, stdout):
-        m_subp.return_value = (stdout, "")
-        with pytest.raises(exceptions.UserFacingError) as e:
-            system.get_lscpu_arch.__wrapped__()
-            assert e.msg_code == messages.LSCPU_ARCH_PARSE_ERROR.name
-
-
 class TestGetDpkgArch:
     @pytest.mark.parametrize(
         "stdout, expected",
@@ -595,21 +509,32 @@ version,codename,series,created,release,eol,eol-server,eol-esm
 
 
 class TestGetPlatformInfo:
+    @pytest.mark.parametrize(
+        "version, expected_exception",
+        (
+            (
+                "junk",
+                exceptions.ParsingErrorOnOSReleaseFile,
+            ),
+            (
+                "22.04 LTS",
+                exceptions.MissingSeriesOnOSReleaseFile,
+            ),
+        ),
+    )
     @mock.patch("uaclient.system.parse_os_release")
-    def test_get_platform_info_error_no_version(self, m_parse):
+    def test_get_platform_info_error(
+        self, m_parse, version, expected_exception
+    ):
         """get_platform_info errors when it cannot parse os-release."""
-        m_parse.return_value = {"VERSION": "junk"}
-        with pytest.raises(RuntimeError) as excinfo:
+        m_parse.return_value = {"VERSION": version}
+        with pytest.raises(expected_exception):
             # Use __wrapped__ to avoid hitting the
             # lru_cached value across tests
             system.get_platform_info.__wrapped__()
-        expected_msg = (
-            "Could not parse /etc/os-release VERSION: junk (modified to junk)"
-        )
-        assert expected_msg == str(excinfo.value)
 
     @pytest.mark.parametrize(
-        "os_release, arch, kernel, expected",
+        "os_release,arch,kernel,virt,expected",
         [
             (
                 {
@@ -618,6 +543,7 @@ class TestGetPlatformInfo:
                 },
                 "arm64",
                 "kernel-ver1",
+                "lxd",
                 {
                     "arch": "arm64",
                     "distribution": "Ubuntu",
@@ -626,6 +552,7 @@ class TestGetPlatformInfo:
                     "series": "xenial",
                     "type": "Linux",
                     "version": "16.04 LTS (Xenial Xerus)",
+                    "virt": "lxd",
                 },
             ),
             (
@@ -635,6 +562,7 @@ class TestGetPlatformInfo:
                 },
                 "amd64",
                 "kernel-ver2",
+                "none",
                 {
                     "arch": "amd64",
                     "distribution": "Ubuntu",
@@ -643,6 +571,7 @@ class TestGetPlatformInfo:
                     "series": "bionic",
                     "type": "Linux",
                     "version": "18.04 LTS (Bionic Beaver)",
+                    "virt": "none",
                 },
             ),
             (
@@ -652,6 +581,7 @@ class TestGetPlatformInfo:
                 },
                 "arm64",
                 "kernel-ver3",
+                "qemu",
                 {
                     "arch": "arm64",
                     "distribution": "Ubuntu",
@@ -660,6 +590,7 @@ class TestGetPlatformInfo:
                     "series": "jammy",
                     "type": "Linux",
                     "version": "22.04 LTS (Jammy Jellyfish)",
+                    "virt": "qemu",
                 },
             ),
             (
@@ -669,6 +600,7 @@ class TestGetPlatformInfo:
                 },
                 "amd64",
                 "kernel-ver4",
+                "wsl",
                 {
                     "arch": "amd64",
                     "distribution": "Ubuntu",
@@ -677,6 +609,48 @@ class TestGetPlatformInfo:
                     "series": "kinetic",
                     "type": "Linux",
                     "version": "22.10 LTS (Kinetic Kudu)",
+                    "virt": "wsl",
+                },
+            ),
+            (
+                {
+                    "NAME": "Ubuntu",
+                    "VERSION": "22.04 LTS",
+                    "VERSION_CODENAME": "Jammy",
+                },
+                "amd64",
+                "kernel-ver4",
+                "lxd",
+                {
+                    "arch": "amd64",
+                    "distribution": "Ubuntu",
+                    "kernel": "kernel-ver4",
+                    "release": "22.04",
+                    "series": "jammy",
+                    "type": "Linux",
+                    "version": "22.04 LTS",
+                    "virt": "lxd",
+                },
+            ),
+            (
+                {
+                    "NAME": "Ubuntu",
+                    "VERSION": "CORRUPTED",
+                    "VERSION_CODENAME": "Jammy",
+                    "VERSION_ID": "22.04",
+                },
+                "amd64",
+                "kernel-ver4",
+                "lxd",
+                {
+                    "arch": "amd64",
+                    "distribution": "Ubuntu",
+                    "kernel": "kernel-ver4",
+                    "release": "22.04",
+                    "series": "jammy",
+                    "type": "Linux",
+                    "version": "CORRUPTED",
+                    "virt": "lxd",
                 },
             ),
         ],
@@ -684,19 +658,23 @@ class TestGetPlatformInfo:
     @mock.patch("uaclient.system.get_kernel_info")
     @mock.patch("uaclient.system.get_dpkg_arch")
     @mock.patch("uaclient.system.parse_os_release")
+    @mock.patch("uaclient.system.get_virt_type")
     def test_get_platform_info_with_version(
         self,
+        m_get_virt_type,
         m_parse_os_release,
         m_get_dpkg_arch,
         m_get_kernel_info,
         os_release,
         arch,
         kernel,
+        virt,
         expected,
     ):
         m_parse_os_release.return_value = os_release
         m_get_dpkg_arch.return_value = arch
         m_get_kernel_info.return_value = mock.MagicMock(uname_release=kernel)
+        m_get_virt_type.return_value = virt
         assert expected == system.get_platform_info.__wrapped__()
 
 
@@ -856,6 +834,32 @@ class TestShouldReboot:
         )
 
 
+class TestWriteFile:
+    @mock.patch("os.unlink")
+    @mock.patch("os.rename")
+    @mock.patch("os.makedirs")
+    @mock.patch("os.chmod")
+    @mock.patch("tempfile.NamedTemporaryFile")
+    def test_delete_tempfile_on_error(
+        self,
+        m_NamedTemporaryFile,
+        m_chmod,
+        m_makedirs,
+        m_rename,
+        m_unlink,
+    ):
+        test_tmpfile = mock.MagicMock()
+        test_tmpfile.name = "test_tmpfile"
+        m_NamedTemporaryFile.return_value = test_tmpfile
+
+        m_rename.side_effect = Exception()
+
+        with pytest.raises(Exception):
+            system.write_file("test", "test")
+
+        assert [mock.call("test_tmpfile")] == m_unlink.call_args_list
+
+
 class TestSubp:
     def test_raise_error_on_timeout(self, _subp):
         """When cmd exceeds the timeout raises a TimeoutExpired error."""
@@ -951,3 +955,35 @@ class TestSubp:
                 assert log in logs
             else:
                 assert log not in logs
+
+
+class TestGetSystemdJobState:
+    @pytest.mark.parametrize(
+        "systemd_return,expected_return",
+        (
+            ("active", True),
+            ("inactive", False),
+            ("", False),
+            (None, False),
+            ("test", False),
+        ),
+    )
+    @mock.patch("uaclient.system.subp")
+    def test_get_systemd_job_state(
+        self,
+        subp,
+        systemd_return,
+        expected_return,
+    ):
+        subp.return_value = (systemd_return, "")
+        assert expected_return == system.get_systemd_job_state(job_name="test")
+
+    @mock.patch("uaclient.system.subp")
+    def test_systemd_job_state_non_zero(
+        self,
+        subp,
+    ):
+        subp.side_effect = exceptions.ProcessExecutionError(
+            cmd="test", exit_code=3, stdout="inactive", stderr=""
+        )
+        assert False is system.get_systemd_job_state(job_name="test")
