@@ -1,8 +1,5 @@
 """
-Version determination functions
-
-These are in their own file so they can be imported by setup.py before we have
-any of our dependencies installed.
+Client version related functions
 """
 import os.path
 import re
@@ -14,11 +11,11 @@ from uaclient.apt import (
     get_apt_cache_policy_for_package,
     get_apt_cache_time,
 )
-from uaclient.defaults import CANDIDATE_CACHE_PATH, UAC_TMP_PATH
+from uaclient.defaults import CANDIDATE_CACHE_PATH, UAC_RUN_PATH
 from uaclient.exceptions import ProcessExecutionError
 from uaclient.system import subp
 
-__VERSION__ = "27.11.2"
+__VERSION__ = "27.14.4"
 PACKAGED_VERSION = "@@PACKAGED_VERSION@@"
 
 CANDIDATE_REGEX = r"Candidate: (?P<candidate>.*?)\n"
@@ -33,7 +30,8 @@ def get_version() -> str:
          XX.Y so return the --long version to allow daily build recipes
          to count commit offset from upstream's XX.Y signed tag.
       b. If run in a git-ubuntu pkg repo, upstream tags aren't visible,
-         parse the debian/changelog in that case
+         believe __VERSION__ is correct - there is and MUST always be a
+         test to make sure it matches debian/changelog
     """
     if not PACKAGED_VERSION.startswith("@@PACKAGED_VERSION"):
         return PACKAGED_VERSION
@@ -56,17 +54,19 @@ def get_last_known_candidate() -> Optional[str]:
         not os.path.exists(CANDIDATE_CACHE_PATH)
         or os.stat(CANDIDATE_CACHE_PATH).st_mtime < last_apt_cache_update
     ):
+        candidate_version = None
         try:
             policy = get_apt_cache_policy_for_package("ubuntu-advantage-tools")
             match = re.search(CANDIDATE_REGEX, policy)
             if match:
                 candidate_version = match.group("candidate")
-                os.makedirs(UAC_TMP_PATH, exist_ok=True)
+                os.makedirs(UAC_RUN_PATH, exist_ok=True)
                 with open(CANDIDATE_CACHE_PATH, "w") as f:
                     f.write(candidate_version)
-                os.chmod(CANDIDATE_CACHE_PATH, 0o777)
+                return candidate_version
         except Exception:
-            pass
+            if candidate_version is not None:
+                return candidate_version
 
     try:
         with open(CANDIDATE_CACHE_PATH, "r") as f:
